/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <utmp.h>
#include <pwd.h>
#include <dirent.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <net/if_ppp.h>
#include "nmalloc.h"
#include "defs.h"
#include "misc.h"

int       daemon_mode = 1;
int       running = 1;

int       addrlen = sizeof(struct sockaddr_in);
int       listen_s,ppp_sock;
tkon      kon[MAX_KONS];
int       no_kons = 0;
char      recv_buf[RECV_BUF_LEN];
int       recv_buf_ptr = RECV_BUF_LEN,recv_buf_size = 0;

tus_hr    us_hr;
tus_ll    *us_ll = NULL,**us_ll_tail = &us_ll;
tus_btree *us_btree = NULL;

tum_ll    *um_ll = NULL,**um_ll_tail = &um_ll;

int       timer,old_timer,old_save_time,time_delta;
char      our_hostname[STR_LEN];

int       no_free_dialins;

/***************************************************************************
* E: = error
* W: = Warning
* I: = Information
***************************************************************************/
void log_msg(char stat,char *msg,int srl)
{
  if(TUSF_HR_G_RUN_LEVEL(us_hr.flags) <= srl)
  {
    char str[STR_LEN];
    char *ct;

    ct = ctime((time_t *)&timer);
    if(ct)
      ct[strlen(ct) - 6] = 0;
    sprintf(str,"%s %c%1d: %s",ct,stat,srl,msg);
    if(daemon_mode)
    {
      FILE *fp;

      if((fp = fopen(LOG_FILE,"a")) != NULL)
      {
        fprintf(fp,"%s\n",str);
        fclose(fp);
      }
    }
    else
      printf("%s\n",str);
  }
}

void log_error(char *msg)
{
  char str[STR_LEN];

  sprintf(str,"%s: %s",msg,sys_errlist[errno]);
  log_msg('E',str,1);
}

/***************************************************************************
*
***************************************************************************/
void us_ll_clr(tus_ll *p)
{
  register int x;

  p->flags =
    TUSF_INSTANT_PURGE_DT | TUSF_CLOBBER_MULT_LOGINS;
  p->user[0] = 0;
  p->email[0] = 0;
  p->tt_lim = 0;
  for(x = 0;x < DT_LIM_SIZE;x++)
  {
    p->dt_lim[x].d_fr = -1;
    p->dt_lim[x].d_to = 0;
    p->dt_lim[x].t_fr = 0;
    p->dt_lim[x].t_to = 0;
    p->dt_lim[x].limit = 0;
  }
  p->expiry_date.day = 1;
  p->expiry_date.month = 1;
  p->expiry_date.year = 1970;
  p->nice_factor = 0;
  p->t_ul_lim = 0;
  p->t_dl_lim = 0;
  p->d_ul_lim = 0;
  p->d_dl_lim = 0;
}

void us_hr_clr(tus_hr *p)
{
  p->flags = TUSF_HR_MIN_FREE_DIALIN_EXEMPT;
  TUSF_HR_S_RUN_LEVEL(p->flags,0);
  strcpy(p->admin_email,"root@");
  gethostname((p->admin_email) + 5,STR_GEN_LEN - 5);
  p->sleep_time = 60;
  p->grace_time = 5 * 60;
  p->save_time = 15 * 60;
  p->min_free_dialins = 0;
  p->act_timeout = 30 * 60;
  us_ll_clr(&(p->default_us));
}

/***************************************************************************
*
***************************************************************************/
void kill_daemon(int sig)
{
  running = 0;
}

void init(void)
{
  register int x;

  for(x = 0;x < MAX_KONS;x++)
    kon[x].s = -1;
  us_hr_clr(&us_hr);
  signal(SIGINT,kill_daemon);
  signal(SIGKILL,kill_daemon);
  signal(SIGTERM,kill_daemon);
  signal(SIGHUP,SIG_IGN);
  signal(SIGPIPE,SIG_IGN);
  old_save_time = old_timer = timer = time(NULL);
}

/***************************************************************************
*
***************************************************************************/
void daemonise(void)
{
  void furk(void)
  {
    switch(fork())
    {
      case -1: //error
        perror("fork()");
        exit(0);
      case 0: //child
        break;
      default:
        exit(0);
    }
  }

  if(daemon_mode)
  {
    register int x;

    fflush(NULL);
    furk();
    for(x = 0;x <= getdtablesize();++x)
      close(x);
    setsid();
    furk();
    umask(0);
    chdir("/");
    x = open("/dev/null",O_RDWR);
    dup(x);
    dup(x);
  }
}

/***************************************************************************
****************************************************************************
****************************************************************************
* clobber code
****************************************************************************
****************************************************************************
***************************************************************************/

/***************************************************************************
*
***************************************************************************/
void append_us_btree(tus_ll *entry)
{
  void a(tus_btree **node)
  {
    if(*node != NULL)
    {
      register int r;

      r = strcmp((*node)->entry->user,entry->user);
      if(r < 0)
        a(&((*node)->left));
      if(r > 0)
        a(&((*node)->right));
    }
    else
    {
      *node = NMALLOC(sizeof(tus_btree));
      if(*node != NULL)
      {
        (*node)->entry = entry;
        (*node)->left = NULL;
        (*node)->right = NULL;
      }
    }
  }
  a(&us_btree);
}

void build_us_btree(void)
{
  void b(tus_ll *entry)
  {
    if(entry != NULL)
    {
      append_us_btree(entry);
      b(entry->next);
    }
  }
  b(us_ll);
}

void free_us_btree(tus_btree **node)
{
  if(*node != NULL)
  {
    free_us_btree(&((*node)->left));
    free_us_btree(&((*node)->right));
    NFREE(*node);
    *node = NULL;
  }
}

int append_us(tus_ll *entry)
{
  *us_ll_tail = NMALLOC(sizeof(tus_ll));
  if(*us_ll_tail != NULL)
  {
    memcpy(*us_ll_tail,entry,TUS_LL_SIZE);
    (*us_ll_tail)->next = NULL;
    append_us_btree(*us_ll_tail);
    us_ll_tail = &((*us_ll_tail)->next);
  }
  else
    return 0;
  return 1;
}

tus_ll *find_us(char *user)
{
  tus_ll *s = NULL;

  void f(tus_btree *node)
  {
    if(node != NULL)
    {
      if(node->entry != NULL)
      {
        register int r = -1;

        r = strcmp(node->entry->user,user);
        if(r < 0)
          f(node->left);
        else
          if(r > 0)
            f(node->right);
          else
            s = node->entry;
      }
    }
  }
  f(us_btree);
  return s;
}

tus_ll *find_us2(char *user)
{
  tus_ll *s;

  s = find_us(user);
  if(s == NULL)
    if(us_hr.flags & TUSF_HR_DEFAULT_US_ENABLED)
      s = &(us_hr.default_us);
  return s;
}

void remove_us(char *user)
{
  void r(tus_ll **entry)
  {
    if(*entry != NULL)
    {
      if(!strcmp((*entry)->user,user))
      {
        tus_ll *d;

        d = *entry;
        *entry = (*entry)->next;
        NFREE(d);
      }
      else
        r(&((*entry)->next));
    }
  }
  void rtail(tus_ll **entry)
  {
    if((*entry) != NULL)
      rtail(&((*entry)->next));
    else
      us_ll_tail = entry;
  }

  free_us_btree(&us_btree);
  r(&us_ll);
  rtail(&us_ll);
  build_us_btree();
}

void free_us(void)
{
  void f_ll(tus_ll **entry)
  {
    if(*entry != NULL)
    {
      f_ll(&((*entry)->next));
      NFREE(*entry);
      *entry = NULL;
    }
  }
  f_ll(&us_ll);
  free_us_btree(&us_btree);
  us_ll_tail = &us_ll;
}

/***************************************************************************
*
***************************************************************************/
int append_um(tum_ll *entry)
{
  *um_ll_tail = NMALLOC(sizeof(tum_ll));
  if(*um_ll_tail != NULL)
  {
    memcpy(*um_ll_tail,entry,TUM_LL_SIZE);
    (*um_ll_tail)->next = NULL;
    um_ll_tail = &((*um_ll_tail)->next);
  }
  else
    return 0;
  return 1;
}

tum_ll *find_um(char *user)
{
  tum_ll *s = NULL;

  void f(tum_ll *p)
  {
    if(p != NULL)
    {
      if(!strcmp(p->user,user))
        s = p;
      else
        f(p->next);
    }
  }
  f(um_ll);
  return s;
}

void remove_um(char *user)
{
  void r(tum_ll **entry)
  {
    if(*entry != NULL)
    {
      if(!strcmp((*entry)->user,user))
      {
        tum_ll *d;

        d = *entry;
        *entry = (*entry)->next;
        NFREE(d);
      }
      else
        r(&((*entry)->next));
    }
  }
  void rtail(tum_ll **entry)
  {
    if((*entry) != NULL)
      rtail(&((*entry)->next));
    else
      um_ll_tail = entry;
  }

  r(&um_ll);
  rtail(&um_ll);
}

void free_um(void)
{
  void f(tum_ll **entry)
  {
    if(*entry != NULL)
    {
      f(&((*entry)->next));
      NFREE(*entry);
      *entry = NULL;
    }
  }
  f(&um_ll);
  um_ll_tail = &um_ll;
}

/***************************************************************************
*
***************************************************************************/
int load_us(void)
{
  register int s = 1;
  FILE *fp;
  char *err_mess = "load_us()";

  if((fp = fopen(US_FILE,"rb")) == NULL)
  {
    log_error(err_mess);
    return 0;
  }
  if(fread(&us_hr,sizeof(tus_hr),1,fp) == 1)
  {
    for(;;)
    {
      tus_ll entry;

      if(fread(&entry,TUS_LL_SIZE,1,fp) == 1)
        append_us(&entry);
      else
      {
        if(!feof(fp))
        {
          log_error(err_mess);
          s = 0;
        }
        break;
      }
    }
  }
  else
    log_error(err_mess);
  fclose(fp);
  return s;
}

void save_us(void)
{
  FILE *fp;
  char *err_mess = "save_us()";

  void save_us_ll(tus_ll *entry)
  {
    if(entry != NULL)
      if(fwrite(entry,TUS_LL_SIZE,1,fp) == 1)
        save_us_ll(entry->next);
      else
        log_error(err_mess);
  }

  if((fp = fopen(US_FILE,"wb")) == NULL)
  {
    log_error(err_mess);
    return;
  }
  if(fwrite(&us_hr,sizeof(tus_hr),1,fp) == 1)
    save_us_ll(us_ll);
  else
    log_error(err_mess);
  fclose(fp);
}

/***************************************************************************
*
***************************************************************************/
void init_um(tum_ll *um)
{
  um->no_active = 0;
  um->grace_countdown = -1;
  um->dt_zone_limit = -1;
  um->dt_set_limit = 0;
  um->old_ul = 0;
  um->old_dl = 0;
}

int load_um(void)
{
  register int s = 1;
  FILE *fp;
  char *err_mess = "load_um()";

  if((fp = fopen(UM_FILE,"rb")) == NULL)
  {
    log_error(err_mess);
    return 0;
  }
  for(;;)
  {
    tum_ll entry;

    if(fread(&entry,TUM_LL_SIZE,1,fp) == 1)
    {
      init_um(&entry);
      append_um(&entry);
    }
    else
    {
      if(!feof(fp))
      {
        log_error(err_mess);
        s = 0;
      }
      break;
    }
  }
  fclose(fp);
  return s;
}

void save_um(void)
{
  FILE *fp;
  char *err_mess = "save_um()";

  void save_um_ll(tum_ll *entry)
  {
    if(entry != NULL)
      if(fwrite(entry,TUM_LL_SIZE,1,fp) == 1)
        save_um_ll(entry->next);
      else
        log_error(err_mess);
  }

  if((fp = fopen(UM_FILE,"wb")) == NULL)
  {
    log_error(err_mess);
    return;
  }
  save_um_ll(um_ll);
  fclose(fp);
}

/***************************************************************************
*
***************************************************************************/
int us_hr_locked(void)
{
  register int a;
    
  for(a = 0;a < MAX_KONS;a++)
    if(kon[a].s != -1)
      if(kon[a].flags & TKON_US_HR_LOCKED)
        return 1;
  return 0;
}

int user_locked(char *u)
{
  register int a;

  for(a = 0;a < MAX_KONS;a++)
    if(kon[a].s != -1) 
      if(!strcmp(kon[a].user_lock,u))
        return 1;
  return 0;
}

/***************************************************************************
*
***************************************************************************/
int user_exempt(char *user)
{
  register int x;

  for(x = 0;x < boot_st.exempt_user_size;x++)
    if(!strcmp(boot_st.exempt_user[x],user))
      return 1;
  return 0;
}

/***************************************************************************
*
***************************************************************************/
int compose_message(char *filename,tmlist *mlist,char **buf)
{
  register int status = 0,m_size,s_size;
  FILE *mfp,*sfp;

  if((mfp = fopen(filename,"r")) != NULL)
  {
    if((sfp = fopen(CLOB_SIGNATURE,"r")) != NULL)
    {
      fseek(mfp,0,SEEK_END);
      m_size = ftell(mfp);
      fseek(mfp,0,SEEK_SET);
      fseek(sfp,0,SEEK_END);
      s_size = ftell(sfp);
      fseek(sfp,0,SEEK_SET);
      if((*buf = NMALLOC(m_size + s_size + 1)) != NULL)
      {
        if(fread(*buf,1,m_size,mfp) == m_size)
          if(fread((*buf) + m_size,1,s_size,sfp) == s_size)
          {
            register int e = 0;

            *((*buf) + m_size + s_size) = 0;
            status = 1;
            for(;;)
            {
              char *p;

              if((mlist + e)->mod == NULL)
                break;
              if((p = strstr(*buf,(mlist + e)->mod)) != NULL)
              {
                register int d,f = strlen((mlist + e)->repl);

                for(d = 0;d < strlen((mlist + e)->mod);d++)
                  if(d < f)
                    p[d] = (mlist + e)->repl[d];
                  else
                    p[d] = ' ';
              }
              e++;
            }
          }
        if(!status) NFREE(*buf);
      }
      fclose(sfp);
    }
    fclose(mfp);
  }
  return status;
}

/***************************************************************************
*
***************************************************************************/
void email_user(char *email_addr,char *msg)
{
  FILE *fp;
  char str[STR_LEN];

  strcpy(str,SENDMAIL_PATH);
  strcat(str," ");
  strcat(str,email_addr);
  if((fp = popen(str,"w")) != NULL)
  {
    fprintf(fp,"From: %s\n",us_hr.admin_email);
    fprintf(fp,"%s",msg);
    pclose(fp);
  }
}

#define tty_printf(s,d) write(s,d,strlen(d))

void flash_terminal(char *user,char *msg)
{
  struct utmp *utmp_ptr;

  setutent();
  for(;;)
  {
    utmp_ptr = getutent();
    if(utmp_ptr == NULL)
      break;
    if(utmp_ptr->ut_type == USER_PROCESS)
      if(!strcmp(utmp_ptr->ut_user,user))
      {
        register int fp;
        char str[STR_LEN];

        strcpy(str,"/dev/");
        strcat(str,utmp_ptr->ut_line);
        fp = open(str,O_WRONLY | O_NOCTTY | O_APPEND);
        if(fp != -1)
        {
          tty_printf(fp,"\a\n");
          tty_printf(fp,"--\n");
          sprintf(str,"From: %s\n",us_hr.admin_email);
          tty_printf(fp,str);
          tty_printf(fp,msg);
          close(fp);
        }
        else
        {
          sprintf(str,
            "'%s' terminal couldn't be opened. User may not be warned",
            user);
          log_msg('E',str,1);
        }
      }
  }
  endutent();
}

/***************************************************************************
*
***************************************************************************/
void notify_user(tus_ll *us,tum_ll *um,int type)
{
  char str[STR_LEN] = "",*msg = NULL;
  tmlist *mlist = NULL;
  char tmp_str1[STR_GEN_LEN];
  char tmp_str2[STR_GEN_LEN];
  char tmp_str3[STR_GEN_LEN];
  char tmp_str4[STR_GEN_LEN];
  char tmp_str5[STR_GEN_LEN];
  tmlist general_mlist[2] =
  {
    {"%1______",um->user},
    {NULL,NULL}
  };
  tmlist acct_expired_mlist[3] =
  {
    {"%1______",um->user},
    {"%2________",tmp_str1},
    {NULL,NULL}
  };
  tmlist tt_exceeded_mlist[5] =
  {
    {"%1______",um->user},
    {"%2______",tmp_str1},
    {"%3______",tmp_str2},
    {"%4___",tmp_str3},
    {NULL,NULL}
  };
  tmlist dt_exceeded_mlist[6] =
  {
    {"%1______",um->user},
    {"%2______",tmp_str1},
    {"%3______",tmp_str2},
    {"%4______",tmp_str3},
    {"%5___",tmp_str4},
    {NULL,NULL}
  };
  tmlist tnet_exceeded_mlist[9] =
  {
    {"%1______",um->user},
    {"%2________",tmp_str1},
    {"%3________",tmp_str2},
    {"%4________",tmp_str3},
    {"%5________",tmp_str4},
    {"%6___",tmp_str5},
    {NULL,NULL}
  };
  tmlist act_to_mlist[6] =
  {
    {"%1______",um->user},
    {"%2___",tmp_str1},
    {NULL,NULL}
  };

  switch(type)
  {
    case CLOB_INDEX:
      strcpy(str,CLOB_MESS);
      mlist = general_mlist;
      break;
    case CLOB_MULTIPLE_INDEX:
      strcpy(str,CLOB_MULTPLE_MESS);
      mlist = general_mlist;
      break;
    case ACCT_EXPIRED_INDEX:
      strcpy(str,ACCT_EXPIRED_MESS);
      mlist = acct_expired_mlist;
      compose_date(&(us->expiry_date),tmp_str1);
      break;
    case EXCEEDED_TT_INDEX:
      strcpy(str,EXCEEDED_TT_MESS);
      mlist = tt_exceeded_mlist;
      str_hh_mm(us->tt_lim,tmp_str1);
      str_hh_mm(um->tt_used,tmp_str2);
      str_mm_ss(us_hr.grace_time,tmp_str3);
      break;
    case EXCEEDED_DT_INDEX:
      strcpy(str,EXCEEDED_DT_MESS);
      mlist = dt_exceeded_mlist;
      sprintf(tmp_str1,"%d",um->dt_zone_limit);
      str_hh_mm(um->dt_set_limit,tmp_str2);
      str_hh_mm(um->dt_used,tmp_str3);
      str_mm_ss(us_hr.grace_time,tmp_str4);
      break;
    case EXCEEDED_TNET_INDEX:
      strcpy(str,EXCEEDED_TNET_MESS);
      mlist = tnet_exceeded_mlist;
      sprintf(tmp_str1,"%d",um->t_ul);
      sprintf(tmp_str2,"%d",um->t_dl);
      sprintf(tmp_str3,"%d",us->t_ul_lim);
      sprintf(tmp_str4,"%d",us->t_dl_lim);
      str_mm_ss(us_hr.grace_time,tmp_str5);
      break;
    case EXCEEDED_DNET_INDEX:
      strcpy(str,EXCEEDED_DNET_MESS);
      mlist = tnet_exceeded_mlist;
      sprintf(tmp_str1,"%d",um->d_ul);
      sprintf(tmp_str2,"%d",um->d_dl);
      sprintf(tmp_str3,"%d",us->d_ul_lim);
      sprintf(tmp_str4,"%d",us->d_dl_lim);
      str_mm_ss(us_hr.grace_time,tmp_str5);
      break;
    case UACT_TIMEOUT_INDEX:
      strcpy(str,UACT_TIMEOUT_MESS);
      mlist = act_to_mlist;
      str_hh_mm(us_hr.act_timeout,tmp_str1);
      break;
  }
  if(compose_message(str,mlist,&msg))
  {
    if(us->flags & TUSF_NOTIFY_VIA_EMAIL)
    {
      if(us->email[0])
        email_user(us->email,msg);
      else
        email_user(um->user,msg);
    }
    else
      flash_terminal(um->user,msg);
    NFREE(msg);
  }
}

/***************************************************************************
*
***************************************************************************/
void prep_um(tum_ll *um)
{
  if(um != NULL)
  {
    um->no_active = 0;
    prep_um(um->next);
  }
}

/***************************************************************************
*
***************************************************************************/
void scan_utmp(void)
{
  struct utmp *utmp_ptr;
  char str[STR_LEN];

  setutent();
  for(;;)
  {
    utmp_ptr = getutent();
    if(utmp_ptr == NULL)
      break;
    if(utmp_ptr->ut_type == USER_PROCESS)
      if(!user_exempt(utmp_ptr->ut_user))
      {
        tum_ll *um;

        um = find_um(utmp_ptr->ut_user);
        if(um == NULL)
        {
          tum_ll u;

          strncpy(u.user,utmp_ptr->ut_user,STR_USER_LEN);
          u.last_active = 0;
          u.tt_used = (timer - utmp_ptr->ut_time) - (timer - old_timer);
          u.dt_used = u.tt_used;
          u.t_ul = 0;
          u.t_dl = 0;
          u.d_ul = 0;
          u.d_dl = 0;
          init_um(&u);
          append_um(&u);
          sprintf(str,"'%s' new user",u.user);
          log_msg('I',str,1);

          um = find_um(utmp_ptr->ut_user);
        }
        if(um != NULL)
        {
          struct stat stbuf;

          (um->no_active)++;
          strcpy(str,"/dev/");
          strcat(str,utmp_ptr->ut_line);
          if(!stat(str,&stbuf))
            if(stbuf.st_mtime > um->last_active)
              um->last_active = stbuf.st_mtime;
        }
      }
  }
  endutent();
}

/***************************************************************************
*
***************************************************************************/
void ppp_owner(char *dev,char *user)
{
  DIR *dp;

  user[0] = 0;
  if((dp = opendir(PPPD_RUN_PID_DIR)) != NULL)
  {
    struct dirent *dir_entry;
    FILE *fp;
    char str[STR_LEN + 1];

    for(;;)
    {
      dir_entry = readdir(dp);
      if(dir_entry == NULL)
        break;
      if(strstr(dir_entry->d_name,dev) != NULL)
        break;
    }
    closedir(dp);
    if(dir_entry == NULL)
      return;

    strcpy(str,PPPD_RUN_PID_DIR);
    strcat(str,"/");
    strcat(str,dir_entry->d_name);
    if((fp = fopen(str,"r")) != NULL)
    {
      char pppd_str[STR_LEN_INT + 1];

      fscanf(fp,"%s",pppd_str);
      fclose(fp);

      strcpy(str,"/proc/");
      strcat(str,pppd_str);
      strcat(str,"/status");
      if((fp = fopen(str,"r")) != NULL)
      {
        for(;;)
        {
          if(fgets(str,STR_LEN,fp) == NULL)
            break;
          if(!memcmp(str,"Uid:",4))
          {
            struct passwd *pw;
            int p_uid = -1;
            sscanf(str,"%*s %d",&p_uid);
            pw = getpwuid(p_uid);
            if(pw != NULL)
              strcpy(user,pw->pw_name);
          }
        }
        fclose(fp);
      }
    }
  }
}

/***************************************************************************
*
***************************************************************************/
void get_ppp_stats(void)
{
  FILE *fp;

  if((fp = fopen("/proc/net/dev","r")) != NULL)
  {
    for(;;)
    {
      char str[STR_LEN + 1],dev[STR_USER_LEN + 1];

      if(fgets(str,STR_LEN,fp) == NULL)
        break;
      str[6] = 0;
      sscanf(str,"%s",dev);
      if(!strncmp(dev,"ppp",3))
      {
        char user[STR_GEN_LEN + 1];

        ppp_owner(dev,user);
        if(user != NULL)
        {
          tum_ll *um;

          um = find_um(user);
          if(um != NULL)
            if(user_locked(um->user))
            {
              char str[STR_LEN];

              sprintf(str,"'%s' locked. network traffic monitoring ignored",
                um->user);
              log_msg('W',str,1);
            }
            else
            {
              struct ifreq     ifr;
              struct ppp_stats stats;

              memset(&ifr,0,sizeof(struct ifreq));
              ifr.ifr_ifru.ifru_data = (caddr_t)&stats;
              strcpy(ifr.ifr_ifrn.ifrn_name,dev);
              if(ioctl(ppp_sock,SIOCGPPPSTATS,&ifr) != -1)
              {
                register int d_ul,d_dl;
                d_ul = stats.p.ppp_ibytes - um->old_ul;
                d_dl = stats.p.ppp_obytes - um->old_dl;
                um->old_ul = stats.p.ppp_ibytes;
                um->old_dl = stats.p.ppp_obytes;
                if(d_ul || d_dl)
                  um->last_active = timer;
                if(d_ul < 0) d_ul = 0;
                if(d_dl < 0) d_dl = 0;
                (um->t_ul) += d_ul;
                (um->t_dl) += d_dl;
                (um->d_ul) += d_ul;
                (um->d_dl) += d_dl;
              }
            }
        }
      }
    }
    fclose(fp);
  }
}

/***************************************************************************
*
***************************************************************************/
void dialins_free(void)
{
  no_free_dialins = -1;
  if(!(us_hr.flags & TUSF_HR_MIN_FREE_DIALIN_EXEMPT))
  {
    register int x;
    char str[STR_LEN];

    no_free_dialins = 0;
    for(x = 0;x < boot_st.mfd_tty_t_size;x++)
    {
      struct stat stbuf;

      strcpy(str,"/dev/tty");
      strcat(str,boot_st.mfd_tty_t[x]);
      if(stat(str,&stbuf) == 0)
        if(stbuf.st_uid == boot_st.mfd_tty_owner)
          no_free_dialins++;
    }
    if(no_free_dialins >= us_hr.min_free_dialins)
    {
      sprintf(str,"Low usage (Dialin Stat: %d/%d). "
                  "Users Exempt from daily time monitoring",
        no_free_dialins,us_hr.min_free_dialins);
      log_msg('I',str,0);
    }
  }
}

/***************************************************************************
*
***************************************************************************/
void clobber_user(tum_ll *um)
{
  um->no_active = 0;
  um->grace_countdown = -1;

#ifdef CLOBBER_LOGIN_PROCESSES_ONLY
  setutent();
  for(;;)
  {
    struct utmp *utmp_ptr;

    utmp_ptr = getutent();
    if(utmp_ptr == NULL)
      break;
    if(utmp_ptr->ut_type == USER_PROCESS)
      if(!strcmp(utmp_ptr->ut_user,um->user))
        killpg(utmp_ptr->ut_pid,SIGHUP);
  }
  endutent();
#else
  {
    DIR *dp;
    struct dirent *dir_entry;
    struct stat statbuf;
    struct passwd *pw;
    char str[STR_LEN];

    pw = getpwnam(um->user);
    if(pw != NULL)
      if((dp = opendir("/proc")) != NULL)
      {
        while((dir_entry = readdir(dp)) != NULL)
        {
          int e_pid;

          if(sscanf(dir_entry->d_name,"%d",&e_pid))
          {
            strcpy(str,"/proc/");
            strcat(str,dir_entry->d_name);
            if((lstat(str,&statbuf) != -1) &&
              (pw->pw_uid == statbuf.st_uid))
              kill(e_pid,SIGHUP);
          }
        }
        closedir(dp);
      }
  }
#endif
}

/***************************************************************************
*
***************************************************************************/
void mon_entry(tus_ll *us,tum_ll *um)
{
  struct tm *tm_s;

  tm_s = localtime((time_t *)&timer);
  if(tm_s != NULL)
  {
    char str[STR_LEN];

/*clobber if clobber flag set (user is banned)*/
    if(us->flags & TUSF_CLOBBER)
    {
      notify_user(us,um,CLOB_INDEX);
      sprintf(str,"'%s' banned, Disconnecting",um->user);
      log_msg('I',str,1);
      clobber_user(um);
      return;
    }

/*check for multiple logins*/
    if((um->no_active > 1) && (us->flags & TUSF_CLOBBER_MULT_LOGINS))
    {
      notify_user(us,um,CLOB_MULTIPLE_INDEX);
      sprintf(str,"'%s' Already logged on. Flashed User, Disconnecting",
        um->user);
      log_msg('I',str,1);
      clobber_user(um);
      return;
    }

/*check expiry date*/
    if(!(us->flags & TUSF_EXPT_EXPIRY_DATE))
    {
      register unsigned int a,b;

      a = us->expiry_date.day +
          (us->expiry_date.month * 31) +
          (us->expiry_date.year * 31 * 12);
      b = tm_s->tm_mday +
          ((tm_s->tm_mon + 1) * 31) +
          ((tm_s->tm_year + 1900) * 31 * 12);
      if(b >= a)
      {
        notify_user(us,um,ACCT_EXPIRED_INDEX);
        sprintf(str,"'%s' Account Expired. Flashed User, Disconnecting",
          um->user);
        log_msg('I',str,1);
        clobber_user(um);
        return;
      }
    }

/*check if user is still active*/
    if(!(us_hr.flags & TUSF_HR_NET_ACT_TO_EXEMPT))
      if((timer - um->last_active) >= us_hr.act_timeout)
      {
        notify_user(us,um,UACT_TIMEOUT_INDEX);
        sprintf(str,"'%s' User Activity Timeout. Flashed User, Disconnecting",
          um->user);
        log_msg('I',str,1);
        clobber_user(um);
        return;
      }

/*set niceness*/
    if(!(us->flags & TUSF_EXPT_NICE))
    {
      struct passwd *pw;

      pw = getpwnam(um->user);
      if(pw != NULL)
        setpriority(PRIO_USER,pw->pw_uid,us->nice_factor);
    }

    if(um->grace_countdown == -1)
    {
      register int x;
      int dct[] = {6,0,1,2,3,4,5};

/*set time zone*/
      if(!(us->flags & TUSF_EXPT_DAILY_TIME))
      {
        um->dt_zone_limit = -1;
        um->dt_set_limit = 0;
        for(x = (DT_LIM_SIZE - 1);x >= 0;x--)
          if(us->dt_lim[x].d_fr != -1)
          {
            register int pass = 0;

            if((dct[tm_s->tm_wday] >= us->dt_lim[x].d_fr) &&
               (dct[tm_s->tm_wday] <= us->dt_lim[x].d_to))
              pass++;
            else
              if(us->dt_lim[x].d_fr > us->dt_lim[x].d_to)
                pass++;
            if((tm_s->tm_hour >= us->dt_lim[x].t_fr) &&
               (tm_s->tm_hour <= us->dt_lim[x].t_to))
              pass++;
            else
              if(us->dt_lim[x].t_fr > us->dt_lim[x].t_to)
                pass++;
            if(pass == 2)
            {
              um->dt_zone_limit = x;
              um->dt_set_limit = us->dt_lim[um->dt_zone_limit].limit;
              break;
            }
          }
      }

/*check total time*/
      if(!(us->flags & TUSF_EXPT_TOTAL_TIME))
        if(um->tt_used >= us->tt_lim)
        {
          notify_user(us,um,EXCEEDED_TT_INDEX);
          sprintf(str,"'%s' Exceeded Total Time. Flashed User",um->user);
          log_msg('I',str,1);
          um->grace_countdown = us_hr.grace_time;
          return;
        }

/*check daily time*/
      if(!(us->flags & TUSF_EXPT_DAILY_TIME))
        if(no_free_dialins < us_hr.min_free_dialins)
          if(um->dt_used >= um->dt_set_limit)
          {
            notify_user(us,um,EXCEEDED_DT_INDEX);
            sprintf(str,"'%s' Exceeded Daily Time. Flashed User",um->user);
            log_msg('I',str,1);
            um->grace_countdown = us_hr.grace_time;
            return;
          }

/*check total uploads limit*/
      if(!(us->flags & TUSF_EXPT_TOTAL_UL_LIM))
        if(um->t_ul >= us->t_ul_lim)
        { 
          notify_user(us,um,EXCEEDED_TNET_INDEX);
          sprintf(str,"'%s' Exceeded Total Uploads Limit. Flashed User",
            um->user);
          log_msg('I',str,1);
          um->grace_countdown = us_hr.grace_time;
          return;   
        }

/*check total dnloads limit*/
    if(!(us->flags & TUSF_EXPT_TOTAL_DL_LIM))
      if(um->t_dl >= us->t_dl_lim)
      { 
        notify_user(us,um,EXCEEDED_TNET_INDEX);
        sprintf(str,"'%s' Exceeded Total Downloads Limit. Flashed User",
          um->user);
        log_msg('I',str,1);
        um->grace_countdown = us_hr.grace_time;
        return;   
      }

/*check daily uploads limit*/
    if(!(us->flags & TUSF_EXPT_DAILY_UL_LIM))
      if(um->d_ul >= us->d_ul_lim)
      { 
        notify_user(us,um,EXCEEDED_DNET_INDEX);
        sprintf(str,"'%s' Exceeded Daily Uploads Limit. Flashed User",
          um->user);
        log_msg('I',str,1);
        um->grace_countdown = us_hr.grace_time;
        return;   
      }

/*check daily dnloads limit*/
    if(!(us->flags & TUSF_EXPT_DAILY_DL_LIM))
      if(um->d_dl >= us->d_dl_lim)
      { 
        notify_user(us,um,EXCEEDED_DNET_INDEX);
        sprintf(str,"'%s' Exceeded Daily Downloads Limit. Flashed User",
          um->user);
        log_msg('I',str,1);
        um->grace_countdown = us_hr.grace_time;
        return;   
      }

    }
    else
    {
      um->grace_countdown -= time_delta;
      if(um->grace_countdown < 0)
        um->grace_countdown = 0;

/*if grace has counted down*/
      if(!um->grace_countdown)
      {
        sprintf(str,"'%s' Disconnecting",um->user);
        log_msg('I',str,1);
        clobber_user(um);
        return;
      }
    }
  }
}

void monitor(void)
{
  tum_ll *um;
  register int lf = 0;

  if(us_hr_locked())
  {
    log_msg('W',"us_hr locked. monitoring ignored",1);
    lf = 1;
  }
  dialins_free();
  get_ppp_stats();
  um = um_ll;
  for(;;)
  {
    if(um == NULL)
      break;
    if(user_locked(um->user))
    {
      char str[STR_LEN];

      sprintf(str,"'%s' locked. monitoring for user ignored",um->user);
      log_msg('W',str,1);
    }
    else
    {
      tus_ll *us;

      us = find_us2(um->user);
      if(um->no_active > 0)
      {
        um->tt_used += time_delta;
        um->dt_used += time_delta;
        if(!lf)
          if(us != NULL)
            mon_entry(us,um);
      }
      else
      {
        init_um(um);
        if(us != NULL)
          if(us->flags & TUSF_INSTANT_PURGE_DT)
          {
            um->dt_used = 0;
            um->d_ul = 0;
            um->d_dl = 0;
          }
      }
    }
    um = um->next;
  }
}

/***************************************************************************
****************************************************************************
****************************************************************************
* network code
****************************************************************************
****************************************************************************
***************************************************************************/

/***************************************************************************
* Server specific net functions
***************************************************************************/
void s_close_kon(tkon *k)
{
  char str[STR_LEN];

  sprintf(str,"[%s:%d] END CONNECTION",k->hostname,k->port);
  log_msg('I',str,0);
  close_socket(k->s);
  k->s = -1;
  no_kons--;
}

int s_net_readln(tkon *k,char *str,int max_str_len)
{
  register int str_ptr = 0;

  for(;;)
    if(recv_buf_ptr < recv_buf_size)
    {
      if(recv_buf[recv_buf_ptr] == '\n')
      {
        register int y;

        str[str_ptr] = 0;
        if(str_ptr > 1)
        {
          y = str_ptr - 1;
          if(str[y] == '\r')
            str[y] = 0;
        }
        recv_buf_ptr++;
        return 1;
      }
      else
      {
        str[str_ptr] = recv_buf[recv_buf_ptr];
        recv_buf_ptr++;
        if(str_ptr < max_str_len)
          str_ptr++;
        else
        {
          str[str_ptr] = 0;
          return 1;
        }
      }
    }
    else
    {
      fd_set fds;
      struct timeval tv = {0,0};

      FD_ZERO(&fds);
      FD_SET(k->s,&fds);
      switch(select(k->s + 1,&fds,NULL,NULL,&tv))
      {
        case -1:
          s_close_kon(k);
        case 0:
          return 0;
        default:
          recv_buf_ptr = 0;
          recv_buf_size = recv(k->s,recv_buf,RECV_BUF_LEN,0);
          switch(recv_buf_size)
          {
            case 0:
              s_close_kon(k);
            case -1:
              return 0;
          }
          break;
      }
    }
}

int s_net_write(tkon *k,char *str)
{
  register int sl;
            
  sl = strlen(str);
  if(send(k->s,str,sl,0) == sl)
    return 1;
  return 0;
}

int s_net_write_cmd(tkon *k,char *str)
{
  register int sl;
            
  sl = strlen(str);
  if(send(k->s,str,sl,0) == sl)
    if(send(k->s,"\n",1,0) == 1)
      return 1;
  return 0;
}

void kill_all_kons(void)
{
  register int a;

  for(a = 0;a < MAX_KONS;a++)
    if(kon[a].s != -1)
      s_close_kon(&(kon[a]));
  close_socket(listen_s);
}

/***************************************************************************
*
***************************************************************************/
void check_new_kons(void)
{
  register int kon_no;

  for(kon_no = 0;kon_no < MAX_KONS;kon_no++)
    if(kon[kon_no].s == -1)
    {
      register int sr;
      fd_set fds;
      struct timeval tv = {0,0};

      FD_ZERO(&fds);
      FD_SET(listen_s,&fds);
      if((sr = select(listen_s + 1,&fds,NULL,NULL,&tv)) == -1)
      {
        log_error("select(), aborting check_new_kons()");
        running = 0;
      }
      else
        if(sr > 0)
        {
          register int cs;
          struct sockaddr_in client_addr_in;
          char str[STR_LEN];

          cs = accept(listen_s,(struct sockaddr *)&client_addr_in,&addrlen);
          if(cs == -1)
          {
            log_error("accept(), aborting check_new_kons()");
            running = 0;
          }
          else
          {
            struct linger ling;
            struct hostent *hp;

            ling.l_onoff  = 1;
            ling.l_linger = 1;
            setsockopt(cs,SOL_SOCKET,SO_LINGER,&ling,sizeof(ling));
            no_kons++;

            kon[kon_no].flags = 0;
            kon[kon_no].mode = TKON_MODE_CMD;
            kon[kon_no].s = cs;
            hp = gethostbyaddr(
              (char *)&client_addr_in.sin_addr,
              sizeof(struct in_addr),
              client_addr_in.sin_family);
            if(hp == NULL)
              strcpy(kon[kon_no].hostname,
                inet_ntoa(client_addr_in.sin_addr));
            else
              strcpy(kon[kon_no].hostname,hp->h_name);
            kon[kon_no].port = ntohs(client_addr_in.sin_port);
            kon[kon_no].time = timer;
            kon[kon_no].user_lock[0] = 0;

            sprintf(str,"[%s:%d] NEW CONNECTION",
              kon[kon_no].hostname,kon[kon_no].port);
            log_msg('I',str,0);
            s_net_write(&(kon[kon_no]),
              "Clobberd " VERSION ", By Jason Nunn <jsno@downunder.net.au>\n");
            s_net_write_cmd(&(kon[kon_no]),cmd_ok);
          }
        }
    }
}

/***************************************************************************
*
***************************************************************************/
char *net_get_param(char *cmd,char *str)
{
  register int si;

  si = strlen(cmd) + 1;
  if(strlen(str) > si)
    return str + si;
  return NULL;
}

/***************************************************************************
*
***************************************************************************/
void auth(tkon *k,char *param)
{
  char str[STR_LEN];

  void fail(void)
  {
    s_net_write_cmd(k,cmd_er);
    sprintf(str,"[%s:%d] cmd %s (failed)",k->hostname,k->port,cmd_pass);
    log_msg('I',str,1);
  }
  if(param != NULL)
    if(!strcmp(boot_st.passwd,param))
    {
      k->flags |= TKON_AUTHORISED;
      s_net_write_cmd(k,cmd_ok);
      sprintf(str,"[%s:%d] cmd %s (authorised)",k->hostname,k->port,cmd_pass);
      log_msg('I',str,0);
    }
    else
      fail();
  else
    fail();
}

/***************************************************************************
*
***************************************************************************/
void get_stats(tkon *k)
{
  char str[STR_LEN];
  register int x;

  for(x = 0;x < MAX_KONS;x++)
    if(kon[x].s != -1)
    {
      sprintf(str,":%-40s %6d %s\n",
        kon[x].hostname,
        kon[x].port,
        kon[x].flags & TKON_AUTHORISED ? "auth" : "");
      s_net_write(k,str);
    }
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_stat);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void get_us_hr(tkon *k)
{
  char str[STR_BIG] = "";

  comp_net_us_hr(&us_hr,str);
  s_net_write(k,str);
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_gushr);
  log_msg('I',str,0);  
}

void lock_us_hr(tkon *k)
{
  char str[STR_LEN];

  if(!us_hr_locked())
  {
    k->flags |= TKON_US_HR_LOCKED;
    s_net_write_cmd(k,cmd_ok);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_lushr);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void get_us_users(tkon *k)
{
  char str[STR_LEN];
  tus_ll *p;

  p = us_ll;
  while(p != NULL)
  {
    sprintf(str,":%s\n",p->user);
    s_net_write(k,str);
    p = p->next;
  }
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_gusu);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void get_us(tkon *k,char *param)
{
  char str[STR_BIG] = "";

  if(param != NULL)
  {
    tus_ll *us;

    us = find_us(param);
    if(us != NULL)
    {
      comp_net_us_ll(us,str);
      s_net_write(k,str);
      s_net_write_cmd(k,cmd_ok);
    }
    else
      s_net_write_cmd(k,cmd_er);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_gus);
  log_msg('I',str,0);
}

void lock_us(tkon *k,char *param)
{
  char str[STR_LEN];

  if(param != NULL)
  {
    if(!user_locked(param))
    {
      strcpy(k->user_lock,param);
      s_net_write_cmd(k,cmd_ok);
    }
    else
      s_net_write_cmd(k,cmd_er);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_luser);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void new_us(tkon *k,char *param)
{
  char str[STR_LEN];
      
  if(param != NULL)
  {
    if(find_us(param) == NULL)
    {
      strcpy(k->user_lock,param);
      s_net_write_cmd(k,cmd_ok);
    }
    else
      s_net_write_cmd(k,cmd_er);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_nus);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void delete_us(tkon *k,char *param)
{
  char str[STR_LEN];
      
  if(param != NULL)
  {
    remove_us(param);
    s_net_write_cmd(k,cmd_ok);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_dus);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void get_um_users(tkon *k)
{
  char str[STR_LEN];
  tum_ll *p;
   
  p = um_ll;
  while(p != NULL)
  {
    sprintf(str,":%s\n",p->user);
    s_net_write(k,str);
    p = p->next;
  }
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_gumu);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void get_um(tkon *k,char *param)
{
  char str[STR_BIG] = "";

  if(param != NULL)
  {
    tum_ll *um;

    um = find_um(param);
    if(um != NULL)
    {
      comp_net_um_ll(um,str);
      s_net_write(k,str);
      s_net_write_cmd(k,cmd_ok);
    }
    else
      s_net_write_cmd(k,cmd_er);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_gum);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void delete_um(tkon *k,char *param)
{
  char str[STR_LEN];
      
  if(param != NULL)
  {
    remove_um(param);
    s_net_write_cmd(k,cmd_ok);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_dum);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void do_report(tkon *k)
{
  FILE *fp;
  char str[STR_LEN];

  if((fp = fopen(REPORT_FILE,"w")) != NULL)
  {
    tum_ll *um;
    int total_users = 0,
      total_ttime = 0,
      total_tul = 0,
      total_tdl = 0,
      total_dtime = 0,
      total_dul = 0,
      total_ddl = 0;

    fprintf(fp,"Clobberd " VERSION " Report."
               "                            Date: %s\n",
      ctime((time_t *)&timer));
    fprintf(fp,
      "User     "
      "    T Time "
      "    T Uploads "
      "    T Dnloads "
      "  D Time "
      " D Uploads "
      " D dnloads\n\n");
    um = um_ll;
    for(;;)
    {
      if(um == NULL)
        break;

      fprintf(fp,"%-8s ",um->user);

      str_hh_mm(um->tt_used,str);
      fprintf(fp,"%10s ",str);
      fprintf(fp,"%13d ",um->t_ul);
      fprintf(fp,"%13d ",um->t_dl);

      str_hh_mm(um->dt_used,str);
      fprintf(fp,"%8s ",str);
      fprintf(fp,"%10d ",um->d_ul);
      fprintf(fp,"%10d",um->d_dl);
      fprintf(fp,"\n");

      total_users++;

      total_ttime += um->tt_used;
      total_tul += um->t_ul;
      total_tdl += um->t_dl;

      total_dtime += um->dt_used;
      total_dul += um->d_ul;
      total_ddl += um->d_dl;

      um = um->next;
    }

    fprintf(fp,"\n%-8d ",total_users);

    str_hh_mm(total_ttime,str);
    fprintf(fp,"%10s ",str);
    fprintf(fp,"%13d ",total_tul);
    fprintf(fp,"%13d ",total_tdl);

    str_hh_mm(total_dtime,str);
    fprintf(fp,"%8s ",str);
    fprintf(fp,"%10d ",total_dul);
    fprintf(fp,"%10d",total_ddl);
    fprintf(fp,"\n");

    fclose(fp);
  }
  else
  {
    sprintf(str,"[%s:%d] Error printing report",k->hostname,k->port);
    log_msg('I',str,1);
  }
}

void total_purge(tkon *k)
{
  char str[STR_LEN];

  do_report(k);
  free_um();
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_tpurg);
  log_msg('I',str,1);
}

void daily_purge(tkon *k)
{
  tum_ll *um;
  char str[STR_LEN];

  um = um_ll;
  for(;;)
  {
    if(um == NULL)
      break;
    um->dt_used = 0;
    um->d_ul = 0;
    um->d_dl = 0;
    um = um->next;
  }
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_dpurg);
  log_msg('I',str,1);
}

void report(tkon *k)
{
  char str[STR_LEN];

  do_report(k);
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_trept);
  log_msg('I',str,1);
}

/***************************************************************************
*
***************************************************************************/
void user_stat(tkon *k,char *param)
{
  char str[STR_LEN];

  void pe(void)
  {
    s_net_write(k,":No such user.\n");
  }

  if(param != NULL)
  {
    tus_ll *us;
    tum_ll *um;

    us = find_us2(param);
    um = find_um(param);
    if(um != NULL)
    {
      char str2[STR_LEN],*str3,str4[STR_LEN];

      str3 = ctime((time_t *)&(um->last_active));
      if(str3 != NULL)
      {
        sprintf(str,":Last active                      : %s",str3);
        s_net_write(k,str);
      }

      str_mm_ss(us_hr.grace_time,str2);
      sprintf(str,":Grace Time                       : %s\n",str2);
      s_net_write(k,str);

      if(us_hr.flags & TUSF_HR_MIN_FREE_DIALIN_EXEMPT)
        strcpy(str2,exempt_label);
      else
        sprintf(str2,"%d",us_hr.min_free_dialins);
      sprintf(str,":Minumum Free Dialings            : %s\n",str2);
      s_net_write(k,str);

      if(us_hr.flags & TUSF_HR_NET_ACT_TO_EXEMPT)
        strcpy(str2,exempt_label);
      else
        str_hh_mm(us_hr.act_timeout,str2);
      sprintf(str,":Activity Timeout                 : %s\n",str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_EXPIRY_DATE)
          strcpy(str2,exempt_label);
        else
          compose_date(&(us->expiry_date),str2);
      sprintf(str,":Expiry Date                      : %s\n",str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_NICE)
          strcpy(str2,exempt_label);
        else
          sprintf(str2,"%d",us->nice_factor);
      sprintf(str,":Niceness                         : %s\n",str2);
      s_net_write(k,str);

      str_hh_mm(um->tt_used,str4);
      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_TOTAL_TIME)
          strcpy(str2,exempt_label);
        else
          str_hh_mm(us->tt_lim,str2);
      sprintf(str,":Total Time                       : %s (Limit: %s)\n",
        str4,str2);
      s_net_write(k,str);

      str_hh_mm(um->dt_used,str4);
      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_DAILY_TIME)
          strcpy(str2,exempt_label);
        else
          if(um->dt_zone_limit == -1)
            strcpy(str2,na_label);
          else
            str_hh_mm(um->dt_set_limit,str2);
      sprintf(str,":Daily Time                       : %s (Limit: %s)\n",
        str4,str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_TOTAL_UL_LIM)
          strcpy(str2,exempt_label);
        else
          sprintf(str2,"%d bytes",us->t_ul_lim);
      sprintf(str,":Total Uploads                    : %d bytes (Limit: %s)\n",
        um->t_ul,str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_TOTAL_DL_LIM)
          strcpy(str2,exempt_label);
        else
          sprintf(str2,"%d bytes",us->t_dl_lim);
      sprintf(str,":Total Dnloads                    : %d bytes (Limit: %s)\n",
        um->t_dl,str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_DAILY_UL_LIM)
          strcpy(str2,exempt_label);
        else
          sprintf(str2,"%d bytes",us->d_ul_lim);
      sprintf(str,":Daily Uploads                    : %d bytes (Limit: %s)\n",
        um->d_ul,str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_DAILY_DL_LIM)
          strcpy(str2,exempt_label);
        else
          sprintf(str2,"%d bytes",us->d_dl_lim);
      sprintf(str,":Daily Dnloads                    : %d bytes (Limit: %s)\n",
        um->d_dl,str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        strcpy(str2,us->flags & TUSF_INSTANT_PURGE_DT ? yes_label : no_label);
      sprintf(str,":Purge Daily Settings on login    : %s\n",str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        strcpy(str2,us->flags & TUSF_CLOBBER_MULT_LOGINS ? yes_label : no_label);
      sprintf(str,":Clobber multiple logins          : %s\n",str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        strcpy(str2,us->flags & TUSF_CLOBBER ? yes_label : no_label);
      sprintf(str,":Banned                           : %s\n",str2);
      s_net_write(k,str);
    }
    else
      pe();
  }
  else
    pe();
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_ustat);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void cmd_mode(tkon *k,char *ins)
{
  char str[STR_LEN];

  if(!strcmp(ins,cmd_quit))
  {
    sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_quit);
    log_msg('I',str,0);
    s_net_write_cmd(k,cmd_ok);
    s_close_kon(k);
  }
  else if(!strncmp(ins,cmd_pass,strlen(cmd_pass)))
    auth(k,net_get_param(cmd_pass,ins));
  else
  {
    register int unk = 0;

    if(k->flags & TKON_AUTHORISED)
    {
      if(!strcmp(ins,cmd_stat))
        get_stats(k);
/*hr*/
      else if(!strcmp(ins,cmd_gushr))
        get_us_hr(k);
      else if(!strcmp(ins,cmd_sushr))
      {
        k->mode = TKON_MODE_S_US_HR;
        k->gp_accum = 0;
        sprintf(str,"[%s:%d] cmd %s (start)",k->hostname,k->port,cmd_sushr);
        log_msg('I',str,0);
      }
      else if(!strcmp(ins,cmd_lushr))
        lock_us_hr(k);
      else if(!strcmp(ins,cmd_uushr))
      {
        k->flags &= ~TKON_US_HR_LOCKED;
        s_net_write_cmd(k,cmd_ok);
        sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_uushr);
        log_msg('I',str,0);
      }
/*us*/
      else if(!strcmp(ins,cmd_gusu))
        get_us_users(k);
      else if(!strncmp(ins,cmd_gus,strlen(cmd_gus)))
        get_us(k,net_get_param(cmd_gus,ins));
      else if(!strcmp(ins,cmd_sus))
      {
        k->mode = TKON_MODE_S_US;
        k->gp_accum = 0;
        sprintf(str,"[%s:%d] cmd %s (start)",k->hostname,k->port,cmd_sus);
        log_msg('I',str,0);
      }
      else if(!strncmp(ins,cmd_luser,strlen(cmd_luser)))
        lock_us(k,net_get_param(cmd_luser,ins));
      else if(!strcmp(ins,cmd_uuser))
      {
        k->user_lock[0] = 0;
        s_net_write_cmd(k,cmd_ok);
        sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_uuser);
        log_msg('I',str,0);
      }
      else if(!strncmp(ins,cmd_nus,strlen(cmd_nus)))
        new_us(k,net_get_param(cmd_nus,ins));
      else if(!strncmp(ins,cmd_dus,strlen(cmd_dus)))
        delete_us(k,net_get_param(cmd_dus,ins));
/*um*/
      else if(!strcmp(ins,cmd_gumu))
        get_um_users(k);
      else if(!strncmp(ins,cmd_gum,strlen(cmd_gum)))
        get_um(k,net_get_param(cmd_gum,ins));
      else if(!strcmp(ins,cmd_sum))
      {
        k->mode = TKON_MODE_S_UM;
        k->gp_accum = 0;
        sprintf(str,"[%s:%d] cmd %s (start)",k->hostname,k->port,cmd_sum);
        log_msg('I',str,0);
      }
      else if(!strncmp(ins,cmd_dum,strlen(cmd_dum)))
        delete_um(k,net_get_param(cmd_dum,ins));
/****/
      else if(!strcmp(ins,cmd_stime))
      {
        sprintf(str,":%x\n",us_hr.sleep_time);
        s_net_write(k,str);
        s_net_write_cmd(k,cmd_ok);
        sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_stime);
        log_msg('I',str,0);
      }
      else if(!strcmp(ins,cmd_tpurg))
        total_purge(k);
      else if(!strcmp(ins,cmd_dpurg))
        daily_purge(k);
      else if(!strcmp(ins,cmd_trept))
        report(k);
      else if(!strncmp(ins,cmd_ustat,strlen(cmd_ustat)))
        user_stat(k,net_get_param(cmd_ustat,ins));
      else if(!strcmp(ins,cmd_die))
      {
        running = 0;
        s_net_write_cmd(k,cmd_ok);
        sprintf(str,"[%s:%d] cmd %s, Server ending",k->hostname,k->port,cmd_die);
        log_msg('I',str,0);
      }
      else
        unk = 1;
    }
    else
      unk = 1;
    if(unk)
    {
      s_net_write_cmd(k,cmd_er);
      sprintf(str,"[%s:%d] cmd unknown",k->hostname,k->port);
      log_msg('I',str,0);
    }
  }
}

/***************************************************************************
*
***************************************************************************/
tus_hr tmp_us_hr;

void set_us_hr_mode(tkon *k,char *ins)
{
  if(!strcmp(ins,cmd_ok))
  {
    memcpy(&us_hr,&tmp_us_hr,sizeof(tus_hr));
    k->mode = TKON_MODE_CMD;
    s_net_write_cmd(k,cmd_ok);
    sprintf(ins,"[%s:%d] cmd %s (finish)",k->hostname,k->port,cmd_sushr);
    log_msg('I',ins,0);
  }
  else
  {
    decomp_net_us_hr(&tmp_us_hr,k->gp_accum,ins);
    (k->gp_accum)++;
  }
}

/***************************************************************************
*
***************************************************************************/
tus_ll tmp_us;

void set_us_mode(tkon *k,char *ins)
{
  void sucess(void)
  {
    s_net_write_cmd(k,cmd_ok);
    sprintf(ins,"[%s:%d] cmd %s (finish)",k->hostname,k->port,cmd_sus);
    log_msg('I',ins,0);
  }

  if(!strcmp(ins,cmd_ok))
  {
    tus_ll *u;

    u = find_us(tmp_us.user);
    if(u == NULL)
    {
      if(append_us(&tmp_us))
        sucess();
      else
      {
        s_net_write_cmd(k,cmd_er);
        sprintf(ins,"[%s:%d] cmd %s (failed)",k->hostname,k->port,cmd_sus);
        log_error(ins);
      }
    }
    else
    {
      memcpy(u,&tmp_us,TUS_LL_SIZE);
      sucess();
    }
    k->mode = TKON_MODE_CMD;
  }  
  else  
  {
    decomp_net_us_ll(&tmp_us,k->gp_accum,ins);
    (k->gp_accum)++;
  }
}

/***************************************************************************
*
***************************************************************************/
tum_ll tmp_um;

void set_um_mode(tkon *k,char *ins)
{
  if(!strcmp(ins,cmd_ok))
  {
    tum_ll *u;

    u = find_um(tmp_um.user);
    if(u != NULL)
    {
      u->grace_countdown = tmp_um.grace_countdown;
      u->tt_used = tmp_um.tt_used;
      u->dt_used = tmp_um.dt_used;
      u->t_ul = tmp_um.t_ul;
      u->t_dl = tmp_um.t_dl;
      u->d_ul = tmp_um.d_ul;
      u->d_dl = tmp_um.d_dl;
      s_net_write_cmd(k,cmd_ok);
      sprintf(ins,"[%s:%d] cmd %s (finish)",k->hostname,k->port,cmd_sum);
      log_msg('I',ins,0);
    }
    else
    {
      s_net_write_cmd(k,cmd_er);
      sprintf(ins,"[%s:%d] cmd %s (failed)",k->hostname,k->port,cmd_sum);
      log_error(ins);
    }
    k->mode = TKON_MODE_CMD;
  }  
  else  
  {
    decomp_net_um_ll(&tmp_um,k->gp_accum,ins);
    (k->gp_accum)++;
  }
}

/***************************************************************************
*
***************************************************************************/
void service_kons(void)
{
  register int kon_no;

  for(kon_no = 0;kon_no < MAX_KONS;kon_no++)
  {
    register int run;

    for(run = 0;run < 256;run++)
      if(kon[kon_no].s != -1)
      {
        char ins[STR_LEN];

        if(s_net_readln(&(kon[kon_no]),ins,STR_LEN_LN) > 0)
        {
          kon[kon_no].time = timer;

          switch(kon[kon_no].mode)
          {
            case TKON_MODE_CMD:
              cmd_mode(&(kon[kon_no]),ins);
              break;
            case TKON_MODE_S_US_HR:
              set_us_hr_mode(&(kon[kon_no]),ins);
              break;
            case TKON_MODE_S_US:
              set_us_mode(&(kon[kon_no]),ins);
              break;
            case TKON_MODE_S_UM:
              set_um_mode(&(kon[kon_no]),ins);
              break;
          }
        }
        else
        {
          if((timer - kon[kon_no].time) > KON_TIMEOUT)
          {
            s_net_write_cmd(&(kon[kon_no]),cmd_er);
            sprintf(ins,"[%s:%d] connection timeout",
              kon[kon_no].hostname,kon[kon_no].port);
            log_msg('I',ins,1);
            s_close_kon(&(kon[kon_no]));
          }
          break;
        }
      }
  }
}

/***************************************************************************
****************************************************************************
****************************************************************************
* main
****************************************************************************
****************************************************************************
***************************************************************************/

/***************************************************************************
*
***************************************************************************/
void rt_loop(void)
{
  while(running)
  {
    register int rt_loop_time;

    rt_loop_time = time(NULL);
    check_new_kons();
    service_kons();
    if((rt_loop_time - old_save_time) > us_hr.save_time)
    {
      old_save_time = rt_loop_time;
      save_um();
      log_msg('I',"Saved um (scheduled save)",0);
    }
    if((rt_loop_time - timer) >= us_hr.sleep_time)
      break;
    if(!no_kons)
      sleep(1);
    else
      usleep(20000);
  }
}

/***************************************************************************
*
***************************************************************************/
void main_loop(void)
{
  while(running)
  {
    old_timer = timer;
    timer = time(NULL);
    time_delta = timer - old_timer;

    prep_um(um_ll);
    scan_utmp();
    monitor();
    rt_loop();
  }
  save_us();
  save_um();
}

/***************************************************************************
*
***************************************************************************/
int main(int argc,char *argv[])
{
  register int x;
  char str[STR_LEN];
  struct passwd *pw;

  printf(WELCOME);
  pw = getpwuid(getuid());
  if(pw != NULL)
    if(strcmp(pw->pw_name,"root"))
    {
      printf("Must be root to run clobberd. Aborting process.\n");
      return -1;
    }
  for(x = 0;x < argc;x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'h':
          printf(
            "\nUsage: clobberd <options>\n"
             " -h        Display this help info\n"
             " -!        Run as normal process, logs sent to screen.\n"
             "\n"
          );
          return 1;
        case '!':
          daemon_mode = 0;
          break;
      }
    }
  }
  sprintf(str,"(%s)",argv[0]);
  if(find_process(str,0) > 0)
  {
    printf("You are already running clobberd. Aborting this process..\a\n");
    return -1;
  }
  init();
  daemonise();
  if(read_boot_conf())
  {
    if(gethostname(our_hostname,STR_LEN) != -1)
    {
      listen_s = server_bind_socket(PORT);
      if(listen_s)
      {
        ppp_sock = socket(AF_INET,SOCK_STREAM,0);
        if(ppp_sock != -1)
        {
          log_msg('I',daemon_started_mess,1);
#ifdef CLOBBER_LOGIN_PROCESSES_ONLY
          log_msg('I',"Will clobber login processes only",1);
#endif
          sprintf(str,"Running on %s:%d and priming..",our_hostname,PORT);
          log_msg('I',str,1);
          log_msg('I',"Loading data files..",1); 
          load_us();
          load_um();
          main_loop();
          free_us();
          free_um();
          kill_all_kons();
          log_msg('I',daemon_ended_mess,1);
          close(ppp_sock);
        }
        else
          log_error("Couldn't creat PPP socket");
      }
      else
      {
        sprintf(str,"Couldn't bind Port %d",PORT);
        log_error(str);
      }
    }
    else
      log_error("Couldn't resolve current hostname");
  }
  else
    log_error("Error reading boot file (read_boot_conf())");
  PRINT_NMEM();
  return 1;
}
