/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
*
****************************************************************************
****************************************************************************
***************************************************************************/
extern "C" {
#include <curses.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>
#include <sys/time.h>
#include <sys/types.h>
#include "nmalloc.h"
#include "defs.h"
#include "misc.h"
}
#include "clobberedit_obj.h"
#include "clobberedit_misc.h"

int       client_s;
FILE      *client_fp = NULL;

tus_hr    us_hr;
tus_ll    us;
tum_ll    um;

char *ddown_mess = "Daemon is down, Can't perform operation.";
char *nerr_mess = "Looks like TCP connection has died in the arse."
                  " Quit out of program and run again.";
char *pwait_mess = "Please wait..";
char *user_nf_mess = "Username not found. Hit any key.";

/***************************************************************************
*
***************************************************************************/
tnotify notify_obj;

void notify(char *title,char *contents)
{
  notify_obj.create(title,contents);
  notify_obj.run();
  notify_obj.dest();
}

void notify_on(char *title,char *contents)
{
  notify_obj.create(title,contents);
  list_obj.remove();
  notify_obj.redraw();
  update();
}

void notify_off(void)
{
  blit_list_obj();
}

/***************************************************************************
*
***************************************************************************/
int net_get_entry(char *str)
{
  register int s;

  if(client_fp == NULL)
    return 0;
  if(fgets(str,STR_LEN_LN,client_fp) == NULL)
  {
    notify(error_label,nerr_mess);
    fclose(client_fp);
    client_fp = NULL;
    return 0;
  }
  s = strlen(str);
  if(s > 0)
    if(str[s - 1] == '\n')
      str[s - 1] = 0;
  return 1;
}

void net_put_entry(char *str)
{
  if(client_fp == NULL)
    return;
  fprintf(client_fp,str);
  fflush(client_fp);
}

void net_put_cmd(char *str)
{
  if(client_fp == NULL)
    return;
  fprintf(client_fp,"%s\n",str);
  fflush(client_fp);
}

/***************************************************************************
*
***************************************************************************/
int net_get_ok(void)
{
  if(client_fp == NULL)
    return 0;
  for(;;)
  {       
    char str[STR_LEN_LN + 1];

    if(net_get_entry(str) == 0)
      return 0;
    if(!strcmp(str,cmd_er))
      return 1;
    if(!strcmp(str,cmd_ok))
      return 2;
  } 
  return 2;
}

/***************************************************************************
*
***************************************************************************/
int net_lock_us_hr(void)
{
  register int s;

  if(client_fp == NULL)
    return 0;
  net_put_cmd(cmd_lushr);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,
      "Daemon is locked by another remote maintanence session."
      " Hit any key.");
  return s;
}

int net_unlock_us_hr(void)
{
  if(client_fp == NULL)
    return 0;
  net_put_cmd(cmd_uushr);
  if(net_get_ok() != 2)
    return 0;
  return 1;
}

int net_get_us_hr(void)
{
  register int c = 0;

  if(client_fp == NULL)
    return 0;
  memset(&us_hr,0,sizeof(tus_hr));
  net_put_cmd(cmd_gushr);
  for(;;)
  {
    char str[STR_LEN_LN];

    if(!net_get_entry(str))
      return 0;
    if(!strcmp(str,cmd_er))
      return 1;
    if(!strcmp(str,cmd_ok))
      return 2;
    decomp_net_us_hr(&us_hr,c,str);
    c++;
  }
  return 0;
}

int net_put_us_hr(void)
{
  char str[STR_BIG] = "";

  if(client_fp == NULL)
    return 0;
  comp_net_us_hr(&us_hr,str);
  net_put_cmd(cmd_sushr);
  net_put_entry(str);
  net_put_cmd(cmd_ok);
  return net_get_ok();
}

/***************************************************************************
*
***************************************************************************/
int net_lock_user(char *user)
{
  register int s;
  char str[STR_LEN];

  if(client_fp == NULL)
    return 0;
  sprintf(str,"%s %s",cmd_luser,user);
  net_put_cmd(str);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,
      "User is locked by another remote maintanence session. Hit any key.");
  return s;
}

int net_unlock_user(void)
{
  if(client_fp == NULL)
    return 0;
  net_put_cmd(cmd_uuser);
  if(net_get_ok() != 2)
    return 0;
  return 1;
}

/***************************************************************************
*
***************************************************************************/
int net_get_us(char *user)
{
  register int c = 0;
  char str[STR_LEN];

  if(client_fp == NULL)
    return 0;
  memset(&us,0,TUS_LL_SIZE);
  sprintf(str,"%s %s",cmd_gus,user);
  net_put_cmd(str);
  for(;;)
  {
    char str[STR_LEN_LN];

    if(!net_get_entry(str))
      return 0;
    if(!strcmp(str,cmd_er))
      return 1;
    if(!strcmp(str,cmd_ok))
      return 2;
    decomp_net_us_ll(&us,c,str);
    c++;
  }
  return 0;
}

int net_put_us(void)
{
  char str[STR_BIG] = "";
  register int s;

  if(client_fp == NULL)
    return 0;
  comp_net_us_ll(&us,str);
  net_put_cmd(cmd_sus);
  net_put_entry(str);
  net_put_cmd(cmd_ok);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,"Error saving User Setting. Hit any key.");
  return s;
}

int net_new_us(char *user)
{
  char str[STR_LEN]; 
  register int s;

  if(client_fp == NULL)
    return 0;
  sprintf(str,"%s %s",cmd_nus,user);
  net_put_cmd(str);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,
      "Username MUST be unique. Didn't add this entry.. Hit any key.");
  return s;
}

int net_delete_us(char *user)
{
  register int s;
  char str[STR_LEN]; 

  if(client_fp == NULL)
    return 0;
  sprintf(str,"%s %s",cmd_dus,user);
  net_put_cmd(str);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,user_nf_mess);
  return s;
}

/***************************************************************************
*
***************************************************************************/
int net_get_um(char *user)
{
  register int c = 0;
  char str[STR_LEN];

  if(client_fp == NULL)
    return 0;
  memset(&um,0,TUM_LL_SIZE);
  sprintf(str,"%s %s",cmd_gum,user);
  net_put_cmd(str);
  for(;;)
  {
    char str[STR_LEN_LN];

    if(!net_get_entry(str))
      return 0;
    if(!strcmp(str,cmd_er))
      return 1;
    if(!strcmp(str,cmd_ok))
      return 2;
    decomp_net_um_ll(&um,c,str);
    c++;
  }
  return 0;
}

int net_put_um(void)
{
  char str[STR_BIG] = "";
  register int s;

  if(client_fp == NULL)
    return 0;
  comp_net_um_ll(&um,str);
  net_put_cmd(cmd_sum);
  net_put_entry(str);
  net_put_cmd(cmd_ok);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,user_nf_mess);
  return s;
}

int net_delete_um(char *user)
{
  register int s;
  char str[STR_LEN]; 

  if(client_fp == NULL)
    return 0;
  sprintf(str,"%s %s",cmd_dum,user);
  net_put_cmd(str);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,user_nf_mess);
  return s;
}

/***************************************************************************
*
***************************************************************************/
int net_get_stime(void)
{
  int s = 60;
  char str[STR_LEN]; 

  if(client_fp != NULL)
  {
    net_put_cmd(cmd_stime);
    if(net_get_entry(str))
    {
      if(str[0] == ':')
        sscanf(str + 1,"%x",&s);
      net_get_ok();
    }
  }
  return s;
}

void net_tpurge(void)
{
  if(client_fp == NULL)
    return;
  net_put_cmd(cmd_tpurg);
  net_get_ok();
}

void net_dpurge(void)
{
  if(client_fp == NULL)
    return;
  net_put_cmd(cmd_dpurg);
  net_get_ok();
}

void net_treport(void)
{
  if(client_fp == NULL)
    return;
  net_put_cmd(cmd_trept);
  net_get_ok();
}

/***************************************************************************
* Edit Daily time limits
***************************************************************************/
class tedit_dt : public tdialog
{
public:
  tdt_lim *dt_lim_ptr;

  tedit_dt(void);
  void hl_s(int x);
  void draw_table();
  void create(tdt_lim *dlp);
  void redraw(void);
  void run(void);
private:
  int  ldpos,hl,sl;
};

tedit_dt::tedit_dt(void)
{
  sl = hl = 0;
}

void tedit_dt::hl_s(int x)
{
  if((hl == ldpos) && (sl == x))
    attron(A_REVERSE);
  else
    attroff(A_REVERSE);
}

void tedit_dt::draw_table(void)
{
  for(ldpos = 0;ldpos < DT_LIM_SIZE;ldpos++)
  {
    register int p;

    attroff(A_REVERSE);
    move(pos_y + 2 + ldpos,pos_x + 3);
    for(p = 0;p < 26;p++) addch(' ');
    move(pos_y + 2 + ldpos,pos_x + 3);
    if((dt_lim_ptr + ldpos)->d_fr == -1)
    {
      if(hl == ldpos) attron(A_REVERSE);
      addstr("--Inactive--");
    }
    else
    {
      char str[STR_LEN_INT];

      hl_s(0);
      addstr(dt_table[(dt_lim_ptr + ldpos)->d_fr]);
      addch(' ');
      hl_s(1);
      addstr(dt_table[(dt_lim_ptr + ldpos)->d_to]);
      addch(' ');
      hl_s(2);
      addstr(tm_table[(dt_lim_ptr + ldpos)->t_fr]);
      addstr(":00 ");
      hl_s(3);
      addstr(tm_table[(dt_lim_ptr + ldpos)->t_to]);
      addstr(":59 ");
      hl_s(4);
      str_hh_mm((dt_lim_ptr + ldpos)->limit,str);
      addstr(str);
    }
  }
}

void tedit_dt::create(tdt_lim *dlp)
{
  dt_lim_ptr = dlp;
  tdialog::create(
    "Edit Daily Time Limits",
    11,33,5,7,
    COLOR_PAIR(4) | A_BOLD);
  tobj::create(this);
}

void tedit_dt::redraw(void)
{
  register int a;

  tdialog::redraw();
  attrset(COLOR_PAIR(5));
  attroff(A_REVERSE);
  move(pos_y + 1,pos_x + 1);
  addstr("Z From To From  To    Limit");
  for(a = 0; a < DT_LIM_SIZE;a++)
  {
    char str[STR_LEN_INT];

    move(pos_y + 2 + a,pos_x + 1);
    sprintf(str,"%d",a);
    addstr(str);
  }
  draw_table();
}

void tedit_dt::run(void)
{
  while(running)
  {
    ch = getch();
    switch(ch)
    {
      case 'Q': //Exit
        running = 0;
        break;
      case KEY_UP:
        if(hl > 0) hl--;
        draw_table();
        update();
        break;
      case KEY_DOWN:
        if(hl < 7) hl++;
        draw_table();
        update();
        break;
      case KEY_LEFT:
        if(sl > 0) sl--;
        draw_table();
        update();
        break;
      case KEY_RIGHT:
        if(sl < 4) sl++;
        draw_table();
        update();
        break;
      case 'E':
        touched = 1;
        (dt_lim_ptr + hl)->d_fr = -1;
        draw_table();
        update();
        break;
      case 10:
        touched = 1;
        if((dt_lim_ptr + hl)->d_fr == -1)
          (dt_lim_ptr + hl)->d_fr = 0;
        else
        {
          (dt_lim_ptr + hl)->limit =
            get_hh_mm(pos_y + 2 + hl,pos_x + 23,8);
          range_check(
            &((dt_lim_ptr + hl)->limit),
            0,24 * 60 * 60);
        }
        draw_table();
        update();
        break;
      case '-':
        touched = 1;
        if((dt_lim_ptr + hl)->d_fr != -1)
        {
          switch(sl)
          {
            case 0:
              if((dt_lim_ptr + hl)->d_fr > 0)
                (dt_lim_ptr + hl)->d_fr--;
              break;
            case 1:
              if((dt_lim_ptr + hl)->d_to > 0)
                (dt_lim_ptr + hl)->d_to--;
              break;
            case 2:
              if((dt_lim_ptr + hl)->t_fr > 0)
                (dt_lim_ptr + hl)->t_fr--;
              break;
            case 3:
              if((dt_lim_ptr + hl)->t_to > 0)
                (dt_lim_ptr + hl)->t_to--;
              break;
          }
          draw_table();
          update();
        }
        break;
      case '=':
        touched = 1;
        if((dt_lim_ptr + hl)->d_fr != -1)
        {
          switch(sl)
          {
            case 0:
              if((dt_lim_ptr + hl)->d_fr < 6)
                (dt_lim_ptr + hl)->d_fr++;
              break;
            case 1:
              if((dt_lim_ptr + hl)->d_to < 6)
                (dt_lim_ptr + hl)->d_to++;
              break;
            case 2:
              if((dt_lim_ptr + hl)->t_fr < 23)
                (dt_lim_ptr + hl)->t_fr++;
              break;
            case 3:
              if((dt_lim_ptr + hl)->t_to < 23)
                (dt_lim_ptr + hl)->t_to++;
              break;
          }
          draw_table();
          update();
        }
        break;
    }
  }
}

/***************************************************************************
* Edit def User Settings
***************************************************************************/
class tedit_def_us : public tdialog
{
public:
  tscroll_select us_select;
  tus_ll         *curr_us;

  tedit_def_us(void);
  virtual ~tedit_def_us(void);
  void set_notify_email(int index);
  void set_tt(int index);
  void set_dt(int index);
  void set_expiry_date(int index);
  void set_niceness(int index);
  void set_tul(int index);
  void set_tdl(int index);
  void set_dul(int index);
  void set_ddl(int index);
  void set_purge_dt(int index);
  void set_clob_mult_logins(int index);
  void set_clobber(int index);
  void create(tus_ll *usp,char *hr,int y,int x);
  void redraw(void);
  void run_selections(void);
  void toggle_exemptions();
  void run(void);
private:
  char str[STR_LEN];
};

tedit_def_us::tedit_def_us(void)
{
}

tedit_def_us::~tedit_def_us(void)
{
}

void tedit_def_us::set_notify_email(int index)
{
  if(curr_us->flags & TUSF_NOTIFY_VIA_EMAIL)
  {
    sprintf(str,"%s",curr_us->email);
    if(strlen(str) > 13) str[12] = '+';  
    str[13] = 0;
    us_select.ammend_str(index,27,str);
  }
  else
    us_select.ammend_str(index,27,"tty");
}

void tedit_def_us::set_tt(int index)
{
  if(curr_us->flags & TUSF_EXPT_TOTAL_TIME)
    us_select.ammend_str(index,27,exempt_label);
  else
  {
    str_hh_mm(curr_us->tt_lim,str);
    us_select.ammend_str(index,27,str);
  }
}

void tedit_def_us::set_dt(int index)
{
  if(curr_us->flags & TUSF_EXPT_DAILY_TIME)
    us_select.ammend_str(index,27,exempt_label);
  else
    us_select.ammend_str(index,27,"");
}

void tedit_def_us::set_expiry_date(int index) //dd:mm:yyyy
{
  if(curr_us->flags & TUSF_EXPT_EXPIRY_DATE)
    us_select.ammend_str(index,27,exempt_label);
  else
  {
    compose_date(&(curr_us->expiry_date),str);
    us_select.ammend_str(index,27,str);
  }
}

void tedit_def_us::set_niceness(int index)
{
  if(curr_us->flags & TUSF_EXPT_NICE)
    us_select.ammend_str(index,27,exempt_label);
  else
  {
    sprintf(str,"%d",curr_us->nice_factor);
    us_select.ammend_str(index,27,str);
  }
}

void tedit_def_us::set_tul(int index)
{
  if(curr_us->flags & TUSF_EXPT_TOTAL_UL_LIM)
    us_select.ammend_str(index,27,exempt_label);
  else
  {
    sprintf(str,"%d",curr_us->t_ul_lim);
    us_select.ammend_str(index,27,str);
  }
}

void tedit_def_us::set_tdl(int index)
{
  if(curr_us->flags & TUSF_EXPT_TOTAL_DL_LIM)
    us_select.ammend_str(index,27,exempt_label);
  else
  {
    sprintf(str,"%d",curr_us->t_dl_lim);
    us_select.ammend_str(index,27,str);
  }
}

void tedit_def_us::set_dul(int index)
{
  if(curr_us->flags & TUSF_EXPT_DAILY_UL_LIM)
    us_select.ammend_str(index,27,exempt_label);
  else
  {
    sprintf(str,"%d",curr_us->d_ul_lim);
    us_select.ammend_str(index,27,str);
  }
}

void tedit_def_us::set_ddl(int index)
{
  if(curr_us->flags & TUSF_EXPT_DAILY_DL_LIM)
    us_select.ammend_str(index,27,exempt_label);
  else
  {
    sprintf(str,"%d",curr_us->d_dl_lim);
    us_select.ammend_str(index,27,str);
  }
}

void tedit_def_us::set_purge_dt(int index)
{
  us_select.ammend_str(index,27,
    curr_us->flags & TUSF_INSTANT_PURGE_DT ? yes_label : no_label);
}

void tedit_def_us::set_clob_mult_logins(int index)
{
  us_select.ammend_str(index,27,
    curr_us->flags & TUSF_CLOBBER_MULT_LOGINS ? yes_label : no_label);
}

void tedit_def_us::set_clobber(int index)
{
  us_select.ammend_str(index,27,
    curr_us->flags & TUSF_CLOBBER ? yes_label : no_label);
}

void tedit_def_us::create(tus_ll *usp,char *hr,int y,int x)
{
  curr_us = usp;
  tdialog::create(hr,14,42,y,x,COLOR_PAIR(4) | A_BOLD);
  us_select.init(pos_y + 1,pos_x + 1,12,40,0);

  us_select.append("Email Address            : xxxxxxxxxxxxx");
  us_select.append("Total Time Limit (hhh:mm): xxxxxx");
  us_select.append("Daily Time Limits        > xxxxxx");
  us_select.append("Expiry Date (dd:mm:yyyy) : dd:mm:yyyy");
  us_select.append("Niceness                 : dddddddddd");
  us_select.append("Total Upload Limit       : dddddddddd");
  us_select.append("Total Download Limit     : dddddddddd");
  us_select.append("Daily Upload Limit       : dddddddddd");
  us_select.append("Daily Download Limit     : dddddddddd");
  us_select.append("Reset DT after Clobber   : xxx");
  us_select.append("Clobber multiple Dialins : xxx");
  us_select.append("Permanently Ban User     : xxx");

  set_notify_email(0);
  set_tt(1);
  set_dt(2);
  set_expiry_date(3);
  set_niceness(4);
  set_tul(5);
  set_tdl(6);
  set_dul(7);
  set_ddl(8);
  set_purge_dt(9);
  set_clob_mult_logins(10);
  set_clobber(11);
  tobj::create(this);
}

void tedit_def_us::redraw(void)
{
  tdialog::redraw();
  attrset(COLOR_PAIR(5));
  us_select.redraw();
}

void tedit_def_us::run_selections(void)
{
  switch(us_select.get_option())
  {
    case 0: // edit email
      if(curr_us->flags & TUSF_NOTIFY_VIA_EMAIL)
      {
        touched = 1;
        us_select.get_string(curr_us->email,27,STR_LEN_LN,13);
        set_notify_email(0);
do_redraw:
        us_select.redraw();
        update();
      }
      break;
    case 1: // get total time
      if(!(curr_us->flags & TUSF_EXPT_TOTAL_TIME))
      {
        touched = 1;
        curr_us->tt_lim = us_select.get_hr_min(27);
        range_check(&(curr_us->tt_lim),0,999 * 60 * 60);
        set_tt(1);
        goto do_redraw;
      }
      break;
    case 2: // Daily Time Limits
      if(!(curr_us->flags & TUSF_EXPT_DAILY_TIME))
      {
        tedit_dt edit_dt;

        edit_dt.create(curr_us->dt_lim);
        edit_dt.run();
        edit_dt.dest();
        if(edit_dt.touched)
          touched = 1;
      }
      break;
    case 3: // Get Expiry Date
      if(!(curr_us->flags & TUSF_EXPT_EXPIRY_DATE))
      {
        int x,y;

        touched = 1;
        str[0] = 0;
        us_select.get_string(str,27,10,10);
        sscanf(str,"%d:%d:%d",&x,&y,&(curr_us->expiry_date.year));
        range_check(&x,1,31);
        range_check(&y,1,12);
        range_check(&(curr_us->expiry_date.year),1970,9999);
        curr_us->expiry_date.day = (char)x;
        curr_us->expiry_date.month = (char)y;
        set_expiry_date(3);
        goto do_redraw;
      }
      break;
    case 4: // Get niceness
      if(!(curr_us->flags & TUSF_EXPT_NICE))
      {
        touched = 1;
        curr_us->nice_factor = us_select.get_integer(27);
        range_check(&(curr_us->nice_factor),-20,20);
        set_niceness(4);
        goto do_redraw;
      }
      break;
    case 5: // tul
      if(!(curr_us->flags & TUSF_EXPT_TOTAL_UL_LIM))
      {
        touched = 1;
        curr_us->t_ul_lim = us_select.get_integer(27);
        set_tul(5);
        goto do_redraw;
      }
      break;
    case 6: // tdl
      if(!(curr_us->flags & TUSF_EXPT_TOTAL_DL_LIM))
      {
        touched = 1;
        curr_us->t_dl_lim = us_select.get_integer(27);
        set_tdl(6);
        goto do_redraw;
      }
      break;
    case 7: // dul
      if(!(curr_us->flags & TUSF_EXPT_DAILY_UL_LIM))
      {
        touched = 1;
        curr_us->d_ul_lim = us_select.get_integer(27);
        set_dul(7);
        goto do_redraw;
      }
      break;
    case 8: // ddl
      if(!(curr_us->flags & TUSF_EXPT_DAILY_DL_LIM))
      {
        touched = 1;
        curr_us->d_dl_lim = us_select.get_integer(27);
        set_ddl(8);
        goto do_redraw;
      }
      break;
    case 9: // toggle purge td on clobber
      touched = 1;
      curr_us->flags ^= TUSF_INSTANT_PURGE_DT;
      set_purge_dt(9);
      goto do_redraw;
    case 10: // toggle clob mult logins
      touched = 1;
      curr_us->flags ^= TUSF_CLOBBER_MULT_LOGINS;
      set_clob_mult_logins(10);
      goto do_redraw;
    case 11: //toggle clobber flag
      touched = 1;
      curr_us->flags ^= TUSF_CLOBBER;
      set_clobber(11);
      goto do_redraw;
  }
}

void tedit_def_us::toggle_exemptions(void)
{
  touched = 1;
  switch(us_select.get_option())
  {
    case 0:
      curr_us->flags ^= TUSF_NOTIFY_VIA_EMAIL;
      set_notify_email(0);
      goto do_redraw;
    case 1:
      curr_us->flags ^= TUSF_EXPT_TOTAL_TIME;
      set_tt(1);
      goto do_redraw;
    case 2:
      curr_us->flags ^= TUSF_EXPT_DAILY_TIME;
      set_dt(2);
      goto do_redraw;
    case 3:
      curr_us->flags ^= TUSF_EXPT_EXPIRY_DATE;
      set_expiry_date(3);
      goto do_redraw;
    case 4:
      curr_us->flags ^= TUSF_EXPT_NICE;
      set_niceness(4);
      goto do_redraw;
    case 5:
      curr_us->flags ^= TUSF_EXPT_TOTAL_UL_LIM;
      set_tul(5);
      goto do_redraw;
    case 6:
      curr_us->flags ^= TUSF_EXPT_TOTAL_DL_LIM;
      set_tdl(6);
      goto do_redraw;
    case 7:
      curr_us->flags ^= TUSF_EXPT_DAILY_UL_LIM;
      set_dul(7);
      goto do_redraw;
    case 8:
      curr_us->flags ^= TUSF_EXPT_DAILY_DL_LIM;
      set_ddl(8);
do_redraw:
      us_select.redraw();
      update();
      break;
  }
}

void tedit_def_us::run(void)
{
  while(running)
  {
    ch = getch();
    switch(ch)
    {
      case 'Q':
        running = 0;
        break;
      case KEY_UP:
        us_select.cursor_up();
        update();
        break;
      case KEY_DOWN:
        us_select.cursor_down();
        update();
        break;
      case 'E':
        toggle_exemptions();
        break;
      case 10:
        run_selections();
        break;
    }
  }
}

/***************************************************************************
* Edit General Settings
***************************************************************************/
class tgen_settings : public tdialog
{
public:
  tscroll_select setting_items;

  tgen_settings(void);
  virtual ~tgen_settings(void);
  void set_mess(char *str,int index);
  void set_run_level(int index);
  void set_sleep_t(int index);
  void set_grace_time(int index);
  void set_clobber_sig(int index);
  void set_min_dialins(int index);
  void set_act_to(int index);
  void set_save_time(int index);
  void set_def_us_enabled(int index);
  int  create(void);
  void redraw(void);
  void run(void);
private:
  char s[STR_LEN];
};

tgen_settings::tgen_settings(void)
{
}

tgen_settings::~tgen_settings(void)
{
}

void tgen_settings::set_mess(char *str,int index)
{
  sprintf(s,"%s",str);
  if(strlen(s) > 40) s[39] = '+';
  s[40] = 0;
  setting_items.ammend_str(index,28,s);
}

void tgen_settings::set_run_level(int index)
{
  switch(TUSF_HR_G_RUN_LEVEL(us_hr.flags))
  {
    case 0:
      setting_items.ammend_str(index,28,verbose_label);
      break;
    case 1:
      setting_items.ammend_str(index,28,normal_label);
      break;
    case 2:
      setting_items.ammend_str(index,28,none_label);
      break;
  }
}

void tgen_settings::set_sleep_t(int index)
{
  str_mm_ss(us_hr.sleep_time,s);
  setting_items.ammend_str(index,28,s);
}

void tgen_settings::set_grace_time(int index)
{
  str_mm_ss(us_hr.grace_time,s);
  setting_items.ammend_str(index,28,s);
}

void tgen_settings::set_min_dialins(int index)
{
  if(us_hr.flags & TUSF_HR_MIN_FREE_DIALIN_EXEMPT)
    setting_items.ammend_str(index,28,exempt_label);
  else
  {
    sprintf(s,"%d",us_hr.min_free_dialins);
    setting_items.ammend_str(index,28,s);
  }
}

void tgen_settings::set_act_to(int index)
{
  if(us_hr.flags & TUSF_HR_NET_ACT_TO_EXEMPT)
    setting_items.ammend_str(index,28,exempt_label);
  else
  {
    str_hh_mm(us_hr.act_timeout,s);
    setting_items.ammend_str(index,28,s);
  }
}

void tgen_settings::set_save_time(int index)
{
  str_hh_mm(us_hr.save_time,s);
  setting_items.ammend_str(index,28,s);
} 

void tgen_settings::set_def_us_enabled(int index)
{ 
  if(us_hr.flags & TUSF_HR_DEFAULT_US_ENABLED)
    setting_items.ammend_str(index,28,yes_label);
  else
    setting_items.ammend_str(index,28,no_label);
}

int tgen_settings::create(void)
{
  notify_on(notify_label,pwait_mess);
  if(net_lock_us_hr() != 2)
  {
    notify_off();
    return 0;
  }
  net_get_us_hr();
  tdialog::create(
    "General Settings",
    11,71,3,3,
    COLOR_PAIR(4) | A_BOLD);
  setting_items.init(pos_y + 1,pos_x + 1,9,68,0);

  setting_items.append(
  "Admin Email Address       :                                         ");
  setting_items.append("Run Level                 :        ");
  setting_items.append("Sleep time of mont (mm:ss):        ");
  setting_items.append("Message Grace (mm:ss)     :        ");
  setting_items.append("Master Save Time (hh:mm)  :        ");
  setting_items.append("Min Free Dialins(DT Exmpt):        ");
  setting_items.append("User Activity T/O (hh:mm) :        ");
  setting_items.append("Default US Setting enabled: xxx");
  setting_items.append("Edit Default US Settings  >");

  set_mess(us_hr.admin_email,0);
  set_run_level(1);
  set_sleep_t(2);
  set_grace_time(3);
  set_save_time(4);
  set_min_dialins(5);
  set_act_to(6);
  set_def_us_enabled(7);

  tobj::create(this);
  return 1;
}

void tgen_settings::redraw(void)
{
  tdialog::redraw();
  attrset(COLOR_PAIR(5));
  setting_items.redraw();
}

void tgen_settings::run(void)
{
  register int x;

  while(running)
  {
    ch = getch();
    switch(ch)
    {
      case 'Q':
        running = 0;
        break;
      case KEY_UP:
        setting_items.cursor_up();
        update();
        break;
      case KEY_DOWN:
        setting_items.cursor_down();
        update();
        break;
      case 'E':
        touched = 1;
        switch(setting_items.get_option())
        {
          case 5:
            us_hr.flags ^= TUSF_HR_MIN_FREE_DIALIN_EXEMPT;
            set_min_dialins(5);
            goto do_redraw;
          case 6:
            us_hr.flags ^= TUSF_HR_NET_ACT_TO_EXEMPT;
            set_act_to(6);
            goto do_redraw;
        }        
        break;
      case 10:
        switch(setting_items.get_option())
        {
          case 0: //Email of Administration
            setting_items.get_string(us_hr.admin_email,28,STR_GEN_LEN,40);
            set_mess(us_hr.admin_email,0);
            goto do_redraw;
          case 1: //Run Level
            touched = 1;
            x = TUSF_HR_G_RUN_LEVEL(us_hr.flags);
            x++;
            if(x > 2) x = 0;
            TUSF_HR_S_RUN_LEVEL(us_hr.flags,x);
            set_run_level(1);
do_redraw:
            setting_items.redraw();
            update();
            break;
          case 2: //Sleep time
            touched = 1;
            us_hr.sleep_time = setting_items.get_min_sec(28);
            range_check(&(us_hr.sleep_time),10,99 * 60);
            set_sleep_t(2);
            goto do_redraw;
          case 3: //Grace Time
            touched = 1;
            us_hr.grace_time = setting_items.get_min_sec(28);
            range_check(&(us_hr.grace_time),0,99 * 60);
            set_grace_time(3);
            goto do_redraw;
          case 4: //save time
            touched = 1;
            us_hr.save_time = setting_items.get_hr_min(28);
            range_check(&(us_hr.save_time),60,99 * 60 * 60);
            set_save_time(4);
            goto do_redraw;
          case 5: //Min Free Dialins
            if(!(us_hr.flags & TUSF_HR_MIN_FREE_DIALIN_EXEMPT))
            {
              touched = 1;
              us_hr.min_free_dialins = setting_items.get_integer(28);
              range_check(&(us_hr.min_free_dialins),0,0x7fffffff);
              set_min_dialins(5);
              goto do_redraw;
            }
            break;
          case 6: //activity timeout value
            if(!(us_hr.flags & TUSF_HR_NET_ACT_TO_EXEMPT))
            {
              touched = 1;
              us_hr.act_timeout = setting_items.get_hr_min(28);
              range_check(&(us_hr.act_timeout),0,0x7fffffff);
              set_act_to(6);
              goto do_redraw;
            }
            break;
          case 7:
            touched = 1;
            us_hr.flags ^= TUSF_HR_DEFAULT_US_ENABLED;
            set_def_us_enabled(7);
            goto do_redraw;
          case 8: //Edit Default User Settings
            {
              tedit_def_us edit_def_us;

              edit_def_us.create(
                &(us_hr.default_us),"Default User Settings",4,5);
              edit_def_us.run();
              edit_def_us.dest();
              if(edit_def_us.touched)
                touched = 1;
            }
            break;
        }
        break;
    }
  }
  notify_on(notify_label,pwait_mess);
  if(touched)
    net_put_us_hr();
  net_unlock_us_hr();
}

/***************************************************************************
* Edit User Settings
***************************************************************************/
class tedit_us : public tdialog
{
public:
  tscroll_select *user_list;
  int            length;

  tedit_us(void);
  virtual ~tedit_us(void);
  int  get_users(char *cmd);
  int  create(void);
  void redraw(void);
  virtual void get_users2(void);
  virtual void select(void);
  virtual void add_user_err(void);
  virtual void add_user(void);
  virtual void delete_user(void);
  void user_search(void);
  virtual void run(void);
private:
};

tedit_us::tedit_us(void)
{
  user_list = NULL;
}

tedit_us::~tedit_us(void)
{
  if(user_list != NULL)
    delete(user_list);
}

int tedit_us::get_users(char *cmd)
{
  if(user_list != NULL)
    delete(user_list);
  user_list = new(tscroll_select);
  if(user_list != NULL)
  {
    user_list->init(pos_y + 1,pos_x + 1,length,8,1);
    net_put_cmd(cmd);
    for(;;)
    {
      char str[STR_LEN_LN + 1];

      if(!net_get_entry(str))
        return 0;
      if(!strcmp(str,cmd_er))
        return 1;
      if(!strcmp(str,cmd_ok))
        break;
      if(str[0] == ':')
        user_list->append(str + 1);
    }
  }
  else
    return 0;
  return 1;
}

int tedit_us::create(void)
{
  tdialog::create(
    "US",
    19,11,3,3,
    COLOR_PAIR(4) | A_BOLD);
  length = 17;
  notify_on(notify_label,pwait_mess);
  if(!get_users(cmd_gusu))
  {
    notify_off();
    return 0;
  }
  tobj::create(this);
  return 1;
}

void tedit_us::redraw(void)
{
  tdialog::redraw();
  attrset(COLOR_PAIR(5));
  user_list->redraw();
}

void tedit_us::get_users2(void)
{
  notify_on(notify_label,pwait_mess);
  if(get_users(cmd_gusu))
  {
    redraw();
    update();
  }
  else
    running = 0;
  notify_off();
}

void tedit_us::select(void)
{
  char *user;

  notify_on(notify_label,pwait_mess);
  user_list->get_str(user_list->get_option(),0,&user);
  if(net_lock_user(user) == 2)
  {
    tedit_def_us edit_us_entry;

    if(net_get_us(user) == 2)
    {
      edit_us_entry.create(&us,"User Settings",3,15);
      edit_us_entry.run();
      edit_us_entry.dest();
      notify_on(notify_label,pwait_mess);
      if(edit_us_entry.touched)
        net_put_us();
    }
    else
      notify(error_label,"Error loading US");
    net_unlock_user();
  }
  notify_off();
}

void tedit_us::add_user_err(void)
{
  notify(error_label,"Bad Username. characters must be between a-z 0-9");
}

void tedit_us::add_user(void)
{
  tstrbox strbox;
  tedit_def_us edit_us;
  char user[STR_USER_LEN + 1] = "";
  register unsigned int a;
  char *acceptable = "abcdefghijklmnopqrstuvwxyz0123456789";

  strbox.create("New User",
    3,STR_USER_LEN,pos_y + 1,pos_x + 2,STR_USER_LEN);
  strbox.run(user);
  strbox.dest();
  if(user[0] != 0)
  {
    for(a = 0;a < strlen(user);a++)
    {
      register int unsigned b,f = 0;

      for(b = 0;b < strlen(acceptable);b++)
        if(user[a] == acceptable[b])
        {
          f = 1;
          break;
        }
      if(!f)
      {
        add_user_err();
        return;
      }
    }
  }
  else
  {
    add_user_err();
    return;
  }
  notify_on(notify_label,pwait_mess);
  if(net_new_us(user) == 2)
  {
    if(net_get_us_hr() == 2)
    {
      memcpy(&us,&(us_hr.default_us),TUS_LL_SIZE);
      strcpy(us.user,user);
      edit_us.create(&us,"User Settings",3,15);
      edit_us.run();
      edit_us.dest();
      notify_on(notify_label,pwait_mess);
      if(net_put_us() == 2)
        if(get_users(cmd_gusu))
        {
          user_list->goto_end();
          redraw();
          update();
        }
    }
    net_unlock_user();
  }
  notify_off();
}

void tedit_us::delete_user(void)
{
  char *user;
  int tmp_pos,tmp_hl;

  tmp_pos = user_list->pos;
  tmp_hl = user_list->hl;
  user_list->get_str(user_list->get_option(),0,&user);
  notify_on(notify_label,pwait_mess);
  if(net_lock_user(user) == 2)
  {
    if(net_delete_us(user) == 2)
      if(get_users(cmd_gusu))
      {
        user_list->pos = tmp_pos;
        user_list->hl = tmp_hl;
        if(user_list->get_option() == user_list->list_size)
          user_list->cursor_up();
        redraw();
        update();
      }
    net_unlock_user();
  }
  notify_off();
}

void tedit_us::user_search(void)
{
  tstrbox u_search;
  char user[STR_USER_LEN + 1] = "";

  u_search.create("Find User",
    3,STR_USER_LEN,pos_y + 1,pos_x + 2,STR_USER_LEN);
  u_search.run(user);
  u_search.dest();
  for(;;)
  {
    register int p;
    char *entry;

    p = user_list->get_option();
    if(p == user_list->list_size)
    {
      notify(notify_label,"Couldn't find user");
      break;
    }
    user_list->get_str(p,0,&entry);
    if(strstr(user,entry) != NULL)
      break;
    user_list->cursor_down();
  }
  redraw();
  update();
}

void tedit_us::run(void)
{
  while(running)
  {
    ch = getch();

    switch(ch)
    {
      case 'Q':
        running = 0;
        break;
      case ' ':
        get_users2();
        break;
      case 'A':
        add_user();
        break;
      default:
        if(user_list->list_size)
          switch(ch)
          {
            case KEY_UP:
              user_list->cursor_up();
              update();
              break;
            case KEY_DOWN:
              user_list->cursor_down();
              update();
              break;
            case 10:
              select();
              break;
            case 'D':
              delete_user();
              break;
            case 'S':
              user_search();
              break;
          }
        break;
    }
  }
}

/***************************************************************************
* Edit User Monitor entry
***************************************************************************/
class tedit_um_entry : public tdialog
{
public:
  tscroll_select um_select;
  tum_ll         *curr_um;

  tedit_um_entry(void);
  virtual ~tedit_um_entry(void);
  void set_no_actives(int index);
  void set_last_act(int index);
  void set_grace_cd(int index);
  void set_ttu(int index);
  void set_dtzl(int index);
  void set_dtsl(int index);
  void set_dtu(int index);
  void set_t_ul(int index);
  void set_t_dl(int index);
  void set_d_ul(int index);
  void set_d_dl(int index);
  void create(tum_ll *ump,char *hr,int y,int x);
  void redraw(void);
  virtual void run(void);
private:
  char str[STR_LEN];
};

tedit_um_entry::tedit_um_entry(void)
{
}

tedit_um_entry::~tedit_um_entry(void)
{
}

void tedit_um_entry::set_no_actives(int index)
{
  sprintf(str,"%d",curr_um->no_active);
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::set_grace_cd(int index)
{
  if(curr_um->grace_countdown != -1)
    str_mm_ss(curr_um->grace_countdown,str);
  else
    strcpy(str,na_label);
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::set_last_act(int index)
{
  char *s;

  s = ctime((const time_t *)&(curr_um->last_active));
  if(s != NULL)
  {
    strcpy(str,s);
    str[strlen(str) - 1] = 0;
  }
  else
    strcpy(str,"ERR");
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::set_ttu(int index)
{
  str_hh_mm(curr_um->tt_used,str);
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::set_dtzl(int index)
{
  if(!curr_um->no_active || (curr_um->dt_zone_limit == -1))
    strcpy(str,na_label);
  else
    sprintf(str,"%d",curr_um->dt_zone_limit);
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::set_dtsl(int index)
{
  if(!curr_um->no_active || (curr_um->dt_zone_limit == -1))
    strcpy(str,na_label);
  else
    str_hh_mm(curr_um->dt_set_limit,str);
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::set_dtu(int index)
{
  str_hh_mm(curr_um->dt_used,str);
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::set_t_ul(int index)
{
  sprintf(str,"%d",curr_um->t_ul);
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::set_t_dl(int index)
{
  sprintf(str,"%d",curr_um->t_dl);
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::set_d_ul(int index)
{
  sprintf(str,"%d",curr_um->d_ul);
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::set_d_dl(int index)
{
  sprintf(str,"%d",curr_um->d_dl);
  um_select.ammend_str(index,27,str);
}

void tedit_um_entry::create(tum_ll *ump,char *hr,int y,int x)
{
  curr_um = ump;
  tdialog::create(hr,13,53,y,x,COLOR_PAIR(4) | A_BOLD);
  um_select.init(pos_y + 1,pos_x + 1,11,51,0);

  um_select.append("No Active                | dddddddddd");
  um_select.append("Grace Countdown          | dddddddddd");
  um_select.append("Last Active              | xxx xxx xx xx:xx:xx xxxx");
  um_select.append("Total Time Used          : dddddddddd");
  um_select.append("Daily Time Login Zone    | dddddddddd");
  um_select.append("Daily Time Set Limit     | dddddddddd");
  um_select.append("Daily Time Used          : dddddddddd");
  um_select.append("Total Uploaded           : dddddddddd");
  um_select.append("Total Downloaded         : dddddddddd");
  um_select.append("Daily Uploaded           : dddddddddd");
  um_select.append("Daily Downloaded         : dddddddddd");

  set_no_actives(0);
  set_grace_cd(1);
  set_last_act(2);
  set_ttu(3);
  set_dtzl(4);
  set_dtsl(5);
  set_dtu(6);
  set_t_ul(7);
  set_t_dl(8);
  set_d_ul(9);
  set_d_dl(10);

  tobj::create(this);
}

void tedit_um_entry::redraw(void)
{
  tdialog::redraw();
  attrset(COLOR_PAIR(5));
  um_select.redraw();
}


void tedit_um_entry::run(void)
{
  while(running)
  {
    ch = getch();
    switch(ch)
    {
      case 'Q':
        running = 0;
        break;
      case KEY_UP:
        um_select.cursor_up();
        update();
        break;
      case KEY_DOWN:
        um_select.cursor_down();
        update();
        break;
      case 10:
        switch(um_select.get_option())
        {
          case 1: //grace_reset
            curr_um->grace_countdown = -1;
            set_grace_cd(1);
            goto do_redraw;
          case 3: //ttu
            curr_um->tt_used = um_select.get_hr_min(27);
            range_check(&(curr_um->tt_used),0,999 * 60 * 60);
            set_ttu(3);
do_redraw:
            um_select.redraw();
            update();
            touched = 1;
            break;
          case 6: //dtu
            curr_um->dt_used = um_select.get_hr_min(27);
            range_check(&(curr_um->dt_used),0,24 * 60 * 60);
            set_dtu(6);
            goto do_redraw;
          case 7: //Total Uploaded
            curr_um->t_ul = um_select.get_integer(27);
            set_t_ul(7);
            goto do_redraw;
          case 8: //Total Downloaded
            curr_um->t_dl = um_select.get_integer(27);
            set_t_dl(8);
            goto do_redraw;
          case 9: //Daily Uploaded
            curr_um->d_ul = um_select.get_integer(27);
            set_d_ul(9);
            goto do_redraw;
          case 10: //Daily Downloaded
            curr_um->d_dl = um_select.get_integer(27);
            set_d_dl(10);
            goto do_redraw;
        }
        break;

    }
  }
}

/***************************************************************************
* Edit User Monitors
***************************************************************************/
class tedit_um : public tedit_us
{
public:

  int  create(void);
  void get_users2(void);
  void select(void);
  void add_user(void);
  void delete_user(void);
private:
};

int tedit_um::create(void)
{
  tdialog::create(
    "UM",
    19,11,3,3,
    COLOR_PAIR(4) | A_BOLD);
  length = 17;
  notify_on(notify_label,pwait_mess);
  if(!get_users(cmd_gumu))
  {
    notify_off();
    return 0;
  }
  tobj::create(this);
  return 1;
}

void tedit_um::get_users2(void)
{
  notify_on(notify_label,pwait_mess);
  if(get_users(cmd_gumu))
  {
    redraw();
    update();
  }
  else
    running = 0;
  notify_off();
}

void tedit_um::select(void)
{
  char *user;

  notify_on(notify_label,pwait_mess);
  user_list->get_str(user_list->get_option(),0,&user);
  if(net_lock_user(user) == 2)
  {
    tedit_um_entry edit_um_entry;

    if(net_get_um(user) == 2)
    {
      edit_um_entry.create(&um,"User Monitor",3,15);
      edit_um_entry.run();
      edit_um_entry.dest();
      notify_on(notify_label,pwait_mess);
      if(edit_um_entry.touched)
        net_put_um();
    }
    else
      notify(error_label,"Error loading UM");
    net_unlock_user();
  }
  notify_off();
}

void tedit_um::add_user(void)
{
  notify(notify_label,"Inapplicable command. No can do. Hit key key.");
}

void tedit_um::delete_user(void)
{
  char *user;
  int tmp_pos,tmp_hl;

  tmp_pos = user_list->pos;
  tmp_hl = user_list->hl;
  user_list->get_str(user_list->get_option(),0,&user);
  notify_on(notify_label,pwait_mess);
  if(net_lock_user(user) == 2)
  {
    if(net_delete_um(user) == 2)
      if(get_users(cmd_gumu))
      {
        user_list->pos = tmp_pos;
        user_list->hl = tmp_hl;
        if(user_list->get_option() == user_list->list_size)
          user_list->cursor_up();
        redraw();
        update();
      }
    net_unlock_user();
  }
  notify_off();
}

/***************************************************************************
* Monitor entry
***************************************************************************/
class tmon_um_entry : public tedit_um_entry
{
public:

  void run(int sleep_time);
};

void tmon_um_entry::run(int sleep_time)
{
  nodelay(stdscr,1);
  running = 0;
  for(;;)
  {
    if((running % 50) == 0)
    {
      char str[10];
      register int i;

      move(LINES - 1,69);
      if(has_colors())
        attrset(COLOR_PAIR(6) | A_BOLD);
      else
        attron(A_REVERSE);
      str_mm_ss(sleep_time - (running / 50),str);
      i = strlen(str);
      str[i] = ' ';
      str[i + 1] = 0;
      addstr(str);
      attrset(COLOR_PAIR(5));
      update();
    }
    usleep(10000);
    switch(getch())
    {
      case 'Q':
        running = -1;
        break;
      case KEY_UP:
        um_select.cursor_up();
        update();
        break;
      case KEY_DOWN:
        um_select.cursor_down();
        update();
        break;
    }
    if(running == -1)
      break;
    else
      if(running < (sleep_time * 50))
        running++;
      else
        break;
  }
  nodelay(stdscr,0);
  if(has_colors())
    attrset(COLOR_PAIR(1) | A_BOLD);
  else
    attron(A_REVERSE);
  move(LINES - 1,69);
  scroll_clear_line(10);
}

/***************************************************************************
* Monitor
***************************************************************************/
class tmon_um : public tedit_um
{
public:

  virtual void select(void);
  virtual void add_user(void);
  virtual void delete_user(void);
private:
};

void tmon_um::select(void)
{
  char *user;
  register int sleep_time,running = 1;

  sleep_time = net_get_stime();
  user_list->get_str(user_list->get_option(),0,&user);
  do
  {
    notify_on(notify_label,pwait_mess);
    if(net_get_um(user) == 2)
    {
      tmon_um_entry mon_um_entry;

      mon_um_entry.create(&um,"User Monitor",3,15);
      mon_um_entry.run(sleep_time);
      running = mon_um_entry.running;
      mon_um_entry.dest();
    }
    else
    {
      notify(error_label,"Error loading UM");
      break;
    }
  }
  while(running != -1);
}

void tmon_um::add_user(void)
{
}

void tmon_um::delete_user(void)
{
}

/***************************************************************************
* connection stats
***************************************************************************/
class tkon_stat : public tdialog
{
public:
  tscroll_select *kon_list;

  tkon_stat(void);
  virtual ~tkon_stat(void);
  int  get_stats(void);
  int  create(void);
  void redraw(void);
  void run(void);
};

tkon_stat::tkon_stat(void)
{
  kon_list = NULL;
}

tkon_stat::~tkon_stat(void)
{
  if(kon_list != NULL)
    delete(kon_list);
}

int tkon_stat::get_stats(void)
{
  if(kon_list != NULL)
    delete(kon_list);
  kon_list = new(tscroll_select);
  if(kon_list != NULL)
  {
    kon_list->init(pos_y + 2,pos_x + 1,17,53,1);
    notify_on(notify_label,pwait_mess);
    net_put_cmd(cmd_stat);
    for(;;)
    {
      char str[STR_LEN_LN + 1];

      if(!net_get_entry(str))
        return 0;
      if(!strcmp(str,cmd_er))
        return 1;
      if(!strcmp(str,cmd_ok))
        break;
      if(str[0] == ':')
        kon_list->append(str + 1);
    }
  }
  else
    return 0;
  return 1;
}

int tkon_stat::create(void)
{
  tdialog::create(
    "Current Connections",
    20,56,2,1,
    COLOR_PAIR(4) | A_BOLD);
  if(!get_stats())
  {
    notify_off();
    return 0;
  }
  tobj::create(this);
  return 1;
}

void tkon_stat::redraw(void)
{
  time_t t;
  char *str_ptr;

  tdialog::redraw();
  t = time(NULL);
  str_ptr = ctime(&t);
  if(str_ptr != NULL)
  {
    addch(' ');
    addnstr(str_ptr + 11,8);
  }
  attrset(COLOR_PAIR(5) | A_BOLD);
  move(pos_y + 1,pos_x + 1);
  addstr("Host");
  move(pos_y + 1,pos_x + 44);
  addstr("Port");
  kon_list->redraw();
}

void tkon_stat::run(void)
{
  while(running)
  {
    ch = getch();
    switch(ch)
    {
      case 'Q':
        running = 0;
        break;
      case KEY_UP:
        kon_list->cursor_up();
        update();
        break;
      case KEY_DOWN:
        kon_list->cursor_down();
        update();
        break;
      case ' ':
        if(get_stats())
        {
          redraw();
          update();
        }
        else
          running = 0;
        break;
    }
  }
}

/***************************************************************************
* Main Menu
***************************************************************************/
class tmain_menu : public tdialog
{
public:
  tscroll_select menu_items;

  tmain_menu(void);
  virtual ~tmain_menu(void);
  void create(void);
  void redraw(void);
  void quit(void);
  int warn(void);
  void run(void);
};

tmain_menu::tmain_menu(void)
{
}

tmain_menu::~tmain_menu(void)
{
}

void tmain_menu::create(void)
{
  tdialog::create(
    "Main Menu",
    12,30,2,1,
    COLOR_PAIR(4) | A_BOLD);
  menu_items.init(pos_y + 1,pos_x + 1,10,28,0);
  menu_items.append("Edit General Settings");        //0
  menu_items.append("Edit User Settings");           //1
  menu_items.append("Edit User Monitors");           //2
  menu_items.append("Monitor");                      //3
  menu_items.append("Daily Purge");                  //4
  menu_items.append("Total Purge (and Report)");     //5
  menu_items.append("Print Report Only");            //6
  menu_items.append("Display Connection stats");     //7
  menu_items.append("Kill the Master and quit");     //8
  menu_items.append("Quit");                         //9
  tobj::create(this);
}

void tmain_menu::redraw(void)
{
  tdialog::redraw();
  attrset(COLOR_PAIR(5));
  menu_items.redraw();
}

void tmain_menu::quit(void)
{
  tquit quit;

  quit.create("Quitting?","Do you wish to Quit (Y/N): ");
  running = quit.run();
  quit.dest();
}

int tmain_menu::warn(void)
{
  tquit quit;
  int s;

  quit.create("Warning!","Are you sure you want to do this? (Y/N): ");
  s = quit.run();
  quit.dest();
  return s;
}

void tmain_menu::run(void)
{
  while(running)
  {
    ch = getch();
    switch(ch)
    {
      case 'Q':
        quit();
        break;
      case KEY_UP:
        menu_items.cursor_up();
        update();
        break;
      case KEY_DOWN:
        menu_items.cursor_down();
        update();
        break;
      case 10:
        switch(menu_items.get_option())
        {
          case 0:
            {
              tgen_settings gen_settings;

              if(gen_settings.create())
              {
                gen_settings.run();
                gen_settings.dest();
              }
            }
            break;
          case 1:
            {
              tedit_us edit_us;

              if(edit_us.create())
              {
                edit_us.run();
                edit_us.dest();
              }
            }
            break;
          case 2:
            {
              tedit_um edit_um;

              if(edit_um.create())
              {
                edit_um.run();
                edit_um.dest();
              }
            }
            break;
          case 3:
            {
              tmon_um mon_um;

              if(mon_um.create())
              {
                mon_um.run();
                mon_um.dest();
              }
            }
            break;
          case 4:
            if(!warn())
            {
              notify_on(notify_label,pwait_mess);
              net_dpurge();
              notify_off();
            }
            break;
          case 5:
            if(!warn())
            {
              notify_on(notify_label,pwait_mess);
              net_tpurge();
              notify_off();
            }
            break;
          case 6:
            notify_on(notify_label,pwait_mess);
            net_treport();
            notify_off();
            break;
          case 7:
            {
              tkon_stat kon_stat;

              if(kon_stat.create())
              {
                kon_stat.run();
                kon_stat.dest();
              }
            }
            break;
          case 8:
            quit();
            if(!running)
            {
              notify_on(notify_label,"B O O O O O M M  ! ! ! !");
              net_put_cmd(cmd_die);
              net_get_ok();
            }
            break;
          case 9:
            quit();
            break;
        }
        break;
    }
  }
}

/***************************************************************************
*
***************************************************************************/
void main_app(void)
{
  notify_on(notify_label,"Connecting to Daemon, please wait...");
  client_s = client_bind_socket(addr,PORT);
  if(client_s)
  {
    client_fp = fdopen(client_s,"a+");
    if(client_fp != NULL)
    {
      if(net_get_ok() == 2)
      {
        tmain_menu main_menu;
        tstrbox strbox;
        char passwd[STR_GEN_LEN + 1];

        display_topbar();
        main_menu.create();
        for(;;)
        {
          register int s;
          char str[STR_LEN_LN];

          passwd[0] = 0;
          strbox.create("Enter Password (or 'quit' to quit)",
            3,STR_GEN_LEN,6,20,STR_GEN_LEN);
          strbox.run(passwd);
          strbox.dest();
          if(!strcmp(passwd,"quit"))
            break;
          notify_on(notify_label,pwait_mess);
          sprintf(str,"%s %s",cmd_pass,passwd);
          net_put_cmd(str);
          s = net_get_ok();
          notify_off();
          if(s == 1)
            notify(error_label,"Password incorrect. Try again.");
          else if(s == 2)
          {
            main_menu.run();
            break;
          }
        }
        main_menu.dest();
        net_put_cmd(cmd_quit);
      }
      if(client_fp != NULL)
        fclose(client_fp);
    }
    close_socket(client_s);
  }
  else
  {
    notify_off();
    notify(error_label,ddown_mess);
  }
}

/***************************************************************************
*
***************************************************************************/
int main(int argc,char *argv[])
{
  printf(WELCOME);
  if(argc != 2)
  {
    printf(
      "Usage: clobberedit <address>\n"
      "\n");
    return 0;
  }
  if(argc == 2)
    addr = argv[1];
  else
  {
    printf("Error: Missing parameters, Aborting.\n");
    return 1;
  }
  client_init();
  initscr();
  start_color();
  init_colour_pairs();
  cbreak();
  noecho();
  keypad(stdscr,TRUE);
  main_app();
  clear();
  refresh();
  resetterm();
  echo();
  endwin();
  PRINT_NMEM();
  return 1;
}
