/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
* clobbertool
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>
#include <sys/time.h>
#include <sys/types.h>
#include "nmalloc.h"
#include "defs.h"
#include "misc.h"

int  client_s;
FILE *client_fp = NULL;

/***************************************************************************
*
***************************************************************************/
int net_get_entry(char *str)
{
  register int s;

  if(client_fp == NULL)
    return 0;
  if(fgets(str,STR_LEN_LN,client_fp) == NULL)
  {
    printf("Network Error, aborting program.\n");
    fclose(client_fp);
    client_fp = NULL;
    return 0;
  }
  s = strlen(str);
  if(s > 0)
    if(str[s - 1] == '\n')
      str[s - 1] = 0;
  return 1;
}

void net_put_entry(char *str)
{
  if(client_fp == NULL)
    return;
  fprintf(client_fp,str);
  fflush(client_fp);
}

void net_put_cmd(char *str)
{
  if(client_fp == NULL)
    return;
  fprintf(client_fp,"%s\n",str);
  fflush(client_fp);
}

int net_get_ok(void)
{
  if(client_fp == NULL)
    return 0;
  for(;;)
  {       
    char str[STR_LEN_LN + 1];

    if(net_get_entry(str) == 0)
      return 0;
    if(!strcmp(str,cmd_er))
      return 1;
    if(!strcmp(str,cmd_ok))
      return 2;
  } 
  return 2;
}

/***************************************************************************
*
***************************************************************************/
void total_purge(void)
{
  printf("Total Purge..\n");
  if(client_fp == NULL)
    return;
  net_put_cmd(cmd_tpurg);
  net_get_ok();
  net_put_cmd(cmd_quit);
}

void daily_purge(void)
{
  printf("Daily Purge..\n");
  if(client_fp == NULL)
    return;
  net_put_cmd(cmd_dpurg);
  net_get_ok();
  net_put_cmd(cmd_quit);
}

/***************************************************************************
*
***************************************************************************/
void user_stat(char *user)
{
  printf("\n");
  if(client_fp != NULL)
  {
    char str[STR_LEN_LN];

    sprintf(str,"%s %s",cmd_ustat,user);
    net_put_cmd(str);
    for(;;)
    {
      if(!net_get_entry(str))
        break;
      if(!strcmp(str,cmd_er))
        break;
      if(!strcmp(str,cmd_ok))
        break;
      printf("%s\n",str + 1);
    }
  }
  printf("\n");
}

/***************************************************************************
*
***************************************************************************/
void print_help(void)
{
  printf(
    DISCLAIMER
    "Usage: clobbertool <options>\n"
    " -h         Display this help info\n"
    " -s <user>  Display User stats\n"
    " -d         Purge Daily Settings\n"
    " -t         Purge Total Settings and Print Report\n"
    "\n");
}

int main(int argc,char *argv[])
{
  register int x,func = 0;
  char *user = "";

  printf(WELCOME);
  if(argc < 2)
  {
    print_help();
    return -1;
  }
  for(x = 1;x < argc;x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'h':
          print_help();
          return 1;
        case 's':
          func = 1;
          if((x + 1) < argc)
            user = argv[x + 1];
          break;
        case 'd':
          func = 2;
          break;
        case 't':
          func = 3;
          break;
      }
    }
  }
  if(!func)
  {
    print_help();
    return -1;
  }
  if(func > 1)
  {
    struct passwd *pw;

    pw = getpwuid(getuid());
    if(pw != NULL)
      if(strcmp(pw->pw_name,"root"))
      {
        printf("Must be root to run clobberd. Aborting process.\n");
        return -1;
      }
  }
  else
    if(strlen(user) > 8)
    {
      printf("Username too long.\n");
      return -1;
    }
  client_init();
  if(read_boot_conf())
  {
    client_s = client_bind_socket(addr,PORT);
    if(client_s)
    {
      printf("Connected to [%s]\n",addr);
      client_fp = fdopen(client_s,"a+");
      if(client_fp != NULL)
      {
        if(net_get_ok() == 2)
        {
          char str[STR_LEN];
          register int s;

          sprintf(str,"%s %s",cmd_pass,boot_st.passwd);
          net_put_cmd(str);
          s = net_get_ok();
          if(s == 1)
            printf("Password incorrect. Aborting program.\n");
          else if(s == 2)
          {
            switch(func)
            {
              case 1:
                user_stat(user);
                break;
              case 2:
                daily_purge();
                break;
              case 3:
                total_purge();
                break;
            }
            printf("Done.\n");
          }
        }
        if(client_fp != NULL)
          fclose(client_fp);
      }
      close_socket(client_s);
    }
    else
      printf("Daemon is down, aborting program.\n");
  }
  else
    perror("read_boot_conf()");
  PRINT_NMEM();
  return 1;
}
