/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
****************************************************************************
****************************************************************************
***************************************************************************/
#define VERSION "4.3"
#define DATED    __DATE__
#define WELCOME \
"Clobberd version " VERSION " (" DATED "), Copyright (C) 1996,1997,1998 Jason Nunn.\n\n"
#define DISCLAIMER \
"Clobberd is under the GNU GENERAL PUBLIC LICENSE AGREEMENT, and\n" \
"comes with ABSOLUTELY NO WARRANTY. This is free software, and you are\n" \
"welcome to redistribute it under certain conditions.\n\n" \
"   Contact: 32 Rothdale Rd, Moil Darwin NT 0810, AUSTRALIA\n" \
"     Email: jsno@downunder.net.au or jsno@stormfront.com.au\n" \
"       WWW: http://www.downunder.net.au/~jsno/\n" \
"\n"

#define STR_LEN             256
#define STR_GEN_LEN         40
#define STR_LEN_LN          80
#define STR_LEN_INT         8
#define STR_LEN_REAL        12
#define STR_USER_LEN        8
#define STR_BIG             1024

#define BOOT_FILE           VAR_PATH "/boot.conf"
#define US_FILE             VAR_PATH "/us." VERSION 
#define UM_FILE             VAR_PATH "/um." VERSION
#define LOG_FILE            VAR_PATH "/log"
#define REPORT_FILE         VAR_PATH "/report"

#define CLOB_SIGNATURE      VAR_PATH "/signature.txt"

#define CLOB_INDEX          0
#define CLOB_MESS           VAR_PATH "/m_banned.txt"
#define CLOB_MULTIPLE_INDEX 1
#define CLOB_MULTPLE_MESS   VAR_PATH "/m_multiple.txt"
#define ACCT_EXPIRED_INDEX  2
#define ACCT_EXPIRED_MESS   VAR_PATH "/m_expired.txt"
#define EXCEEDED_TT_INDEX   3
#define EXCEEDED_TT_MESS    VAR_PATH "/m_total_time.txt"
#define EXCEEDED_DT_INDEX   4
#define EXCEEDED_DT_MESS    VAR_PATH "/m_daily_time.txt"
#define EXCEEDED_TNET_INDEX 5
#define EXCEEDED_TNET_MESS  VAR_PATH "/m_total_net.txt"
#define EXCEEDED_DNET_INDEX 6
#define EXCEEDED_DNET_MESS  VAR_PATH "/m_daily_net.txt"
#define UACT_TIMEOUT_INDEX  7
#define UACT_TIMEOUT_MESS   VAR_PATH "/m_activity_to.txt"

#define RECV_BUF_LEN        512

#if PROBE_MALLOC
  #define NMALLOC    nmalloc
  #define NREALLOC   nrealloc
  #define NFREE      nfree
  #define PRINT_NMEM print_nmem
#else
  #define NMALLOC    malloc
  #define NREALLOC   realloc
  #define NFREE      free
  #define PRINT_NMEM print_nmem_nothing
#endif

/*************************************************/
typedef struct
{
  char *mod;
  char *repl;
} tmlist;

/*************************************************/
#define TTY_T_SIZE       256
#define TTY_N_SIZE       5
#define EXEMPT_USER_SIZE 8

typedef struct
{
  char passwd[STR_GEN_LEN + 1];
  int  mfd_tty_owner;
  int  mfd_tty_t_size;
  char mfd_tty_t[TTY_T_SIZE][TTY_N_SIZE];
  int  exempt_user_size;
  char exempt_user[EXEMPT_USER_SIZE][STR_USER_LEN + 1];
} tboot_st;

/*************************************************/
#define DT_LIM_SIZE                 8

#define TUSF_NOTIFY_VIA_EMAIL       0x00001
#define TUSF_EXPT_TOTAL_TIME        0x00002
#define TUSF_EXPT_DAILY_TIME        0x00004
#define TUSF_EXPT_EXPIRY_DATE       0x00008
#define TUSF_EXPT_NICE              0x00010
#define TUSF_EXPT_TOTAL_UL_LIM      0x00020
#define TUSF_EXPT_TOTAL_DL_LIM      0x00040
#define TUSF_EXPT_DAILY_UL_LIM      0x00080
#define TUSF_EXPT_DAILY_DL_LIM      0x00100
#define TUSF_INSTANT_PURGE_DT       0x00200
#define TUSF_CLOBBER_MULT_LOGINS    0x00400
#define TUSF_CLOBBER                0x00800

typedef struct
{
  char d_fr;            // 0 = Mon, 6 = Sun
  char d_to;
  char t_fr;            // 1am - 12pm
  char t_to;
  int  limit;
} tdt_lim;

typedef struct
{
  char day;
  char month;
  int  year;
} tdate_st;

typedef struct _tus_ll
{
  int            flags;
  char           user[STR_USER_LEN + 1];
  char           email[STR_LEN_LN + 1];
  int            tt_lim;
  tdt_lim        dt_lim[DT_LIM_SIZE];
  tdate_st       expiry_date;
  int            nice_factor;
  int            t_ul_lim;
  int            t_dl_lim;
  int            d_ul_lim;
  int            d_dl_lim;

  struct _tus_ll *next;
} tus_ll;

#define TUS_LL_SIZE (sizeof(tus_ll) - sizeof(tus_ll *))

typedef struct _tus_btree
{
  tus_ll            *entry;
  struct _tus_btree *left;
  struct _tus_btree *right;
} tus_btree;

#define TUSF_HR_G_RUN_LEVEL(n)         (n & 3)
#define TUSF_HR_MIN_FREE_DIALIN_EXEMPT 0x10
#define TUSF_HR_NET_ACT_TO_EXEMPT      0x20
#define TUSF_HR_DEFAULT_US_ENABLED     0x40

#define TUSF_HR_S_RUN_LEVEL(v,n)       v = ((v & ~3) | n)

typedef struct
{
  int    flags;
  char   admin_email[STR_GEN_LEN + 1];
  int    sleep_time;
  int    grace_time;
  int    save_time;
  int    min_free_dialins;
  int    act_timeout;
  tus_ll default_us;
} tus_hr;

/*************************************************/
typedef struct _tum_ll
{
  char           user[STR_USER_LEN + 1];
  int            no_active;
  int            grace_countdown;
  int            last_active;
  int            tt_used;
  int            dt_zone_limit;
  int            dt_set_limit;
  int            dt_used;
  int            t_ul;
  int            t_dl;
  int            d_ul;
  int            d_dl;
  int            old_ul;
  int            old_dl;
  struct _tum_ll *next;
} tum_ll;

#define TUM_LL_SIZE (sizeof(tum_ll) - sizeof(tum_ll *))

/*************************************************/
#define TKON_AUTHORISED   0x1
#define TKON_US_HR_LOCKED 0x2

#define TKON_MODE_CMD       0
#define TKON_MODE_S_US_HR   1
#define TKON_MODE_S_US      2
#define TKON_MODE_S_UM      3

typedef struct
{
  int    flags;
  int    mode;
  int    s;
  char   hostname[STR_GEN_LEN + 1];
  int    port;
  int    time;
  char   user_lock[STR_USER_LEN + 1];
  int    gp_accum;
} tkon;
