/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <signal.h>
#include <pwd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <utmp.h>
#include "defs.h"

char *addr = "localhost";
tboot_st boot_st;

char  *notify_label = "Notify";
char  *warning_label = "Warning";
char  *error_label = "Error";
char  *yes_label = "Yes";
char  *no_label  = "No";
char  *unknown_label = "UNKNOWN";
char  *exempt_label = "Exempt";
char  *na_label = "n/a";
char  *verbose_label = "Verbose";
char  *normal_label = "Normal";
char  *none_label = "None";
char  *daemon_started_mess = "--Daemon Started----";
char  *daemon_ended_mess = "--Daemon ended------";
char  *dt_table[] = {"Mon","Tue","Wed","Thu","Fri","Sat","Sun"};
char  *tm_table[] = {"00","01","02","03","04","05",
                     "06","07","08","09","10","11",
                     "12","13","14","15","16","17",
                     "18","19","20","21","22","23"};

char *cmd_ok     = ";ok";
char *cmd_er     = ";er";

char *cmd_quit   = ";quit";
char *cmd_pass   = ";pass";
char *cmd_stat   = ";stat";

char *cmd_gushr  = ";gushr";
char *cmd_sushr  = ";sushr";
char *cmd_lushr  = ";lushr";
char *cmd_uushr  = ";uushr";

char *cmd_gusu   = ";gusu";
char *cmd_gus    = ";gus";
char *cmd_sus    = ";sus";
char *cmd_nus    = ";nus";
char *cmd_dus    = ";dus";

char *cmd_gumu   = ";gumu";
char *cmd_gum    = ";gum";
char *cmd_sum    = ";sum";
char *cmd_dum    = ";dum";

char *cmd_luser  = ";luser";
char *cmd_uuser  = ";uuser";

char *cmd_stime  = ";stime";

char *cmd_tpurg  = ";tpurg";
char *cmd_dpurg  = ";dpurg";

char *cmd_trept  = ";trept";

char *cmd_ustat  = ";ustat";

char *cmd_die    = ";die";

/***************************************************************************
*
***************************************************************************/
void client_init(void)
{
  signal(SIGPIPE,SIG_IGN);
  signal(SIGINT,SIG_IGN);
}

/***************************************************************************
*
***************************************************************************/
int freadln(FILE *fp,char *str)
{
  str[0] = 0;
  fgets(str,STR_LEN,fp);
  if((str[0] != '#') && (str[0] >= 32)) return 1;
  return 0;
}

int read_boot_conf(void)
{
  FILE *fp;
  char str[STR_LEN + 1],substr[STR_LEN + 1];

  int get_par(char *d_str,char *s_str,char *key,int max)
  {
    register int siz = strlen(key);
    if(!strncmp(s_str,key,strlen(key)))
      if(strlen(s_str) > siz)
      {
        char *ptr;

        strncpy(d_str,s_str + siz + 1,max);
        d_str[max] = 0;
        if((ptr = memchr(d_str,'\n',max)) != NULL)
          *ptr = 0;
        return 1;
      }
    return 0;
  }

  if((fp = fopen(BOOT_FILE,"r")) == NULL)
    return 0;
  boot_st.mfd_tty_t_size = 0;
  boot_st.exempt_user_size = 0;
  while(!feof(fp))
    if(freadln(fp,str))
    {
      get_par(boot_st.passwd,str,"passwd:",STR_GEN_LEN);
      if(get_par(substr,str,"tty_owner:",STR_GEN_LEN))
      {
        struct passwd *pw_st;

        if((pw_st = getpwnam(substr)) != NULL)
          boot_st.mfd_tty_owner = pw_st->pw_uid;
      }
      if(get_par(substr,str,"add_tty:",STR_GEN_LEN))
      {
        if(boot_st.mfd_tty_t_size < TTY_T_SIZE)
        {
          strncpy(boot_st.mfd_tty_t[boot_st.mfd_tty_t_size],
            substr,TTY_N_SIZE);
          boot_st.mfd_tty_t_size++;
        }
      }
      if(get_par(substr,str,"add_exempt_user:",STR_GEN_LEN))
      {
        if(boot_st.exempt_user_size < EXEMPT_USER_SIZE)
        {
          strncpy(boot_st.exempt_user[boot_st.exempt_user_size],
            substr,STR_USER_LEN);
          boot_st.exempt_user_size++;
        }
      }
    }
  fclose(fp);
  return 1;
}

/***************************************************************************
*
***************************************************************************/
void str_mm_ss(int t,char *s)
{
  char ms[4];

  if(t < 0) t = 0;
  sprintf(ms,"%d",(int)(t % 60));
  if(strlen(ms) == 1)
  {
    ms[1] = ms[0];
    ms[0] = '0';
    ms[2] = 0;
  }
  sprintf(s,"%d:%s",(int)(t / 60),ms);
}

void str_hh_mm(int t,char *s)
{
  char ms[4];

  if(t < 0) t = 0;
  t /= 60;
  sprintf(ms,"%d",(int)(t % 60));
  if(strlen(ms) == 1)
  {
    ms[1] = ms[0];
    ms[0] = '0';
    ms[2] = 0;
  }
  sprintf(s,"%d:%s",(int)(t / 60),ms);
}

void range_check(int *val,int lim1,int lim2)
{
  if(*val < lim1) *val = lim1;
  if(*val > lim2) *val = lim2;
}

void compose_date(tdate_st *d,char *str)
{
  register int x;

  sprintf(str,"%2d:%2d:%4d",d->day,d->month,d->year);
  for(x = 0;x < 10;x++)
    if(str[x] == ' ') str[x] = '0';
}

/***************************************************************************
*
***************************************************************************/
int server_bind_socket(int port)
{
  register int s;
  struct sockaddr_in myaddr_in;
  int reuse_opt = 1;

  memset((char *)&myaddr_in,0,sizeof(struct sockaddr_in));
  myaddr_in.sin_family = AF_INET;
  myaddr_in.sin_addr.s_addr = INADDR_ANY;
  myaddr_in.sin_port = htons(port);
  if((s = socket(AF_INET,SOCK_STREAM,0)) != -1)
  {
    setsockopt(s,SOL_SOCKET,SO_REUSEADDR,&reuse_opt,sizeof(reuse_opt));
    if(bind(s,(struct sockaddr *)&myaddr_in,
      sizeof(struct sockaddr_in)) != -1)
      if(listen(s,5) != -1)
        return s;
  }
  return 0;
}

int client_bind_socket(char *addr,int port)
{
  register int s;
  struct hostent *hp;
  struct sockaddr_in saddr_in;

  memset((char *)&saddr_in,0,sizeof(struct sockaddr_in));
  if((hp = gethostbyname(addr)) != NULL)
  {
    saddr_in.sin_family = AF_INET;
    saddr_in.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;
    saddr_in.sin_port = htons(port);
    if((s = socket(AF_INET,SOCK_STREAM,0)) != -1)
      if(connect(s,(struct sockaddr *)&saddr_in,
        sizeof(struct sockaddr_in)) != -1)
        return s;
  }
  return 0;
}

void close_socket(int s)
{
  shutdown(s,2);
  close(s);
}

/***************************************************************************
*
***************************************************************************/
int find_process(char *process_name,int mode)
{
  DIR *dp;
  struct dirent *dir_entry;
  int proc_id = -1;

  if((dp = opendir("/proc")) != NULL)
  {
    while((dir_entry = readdir(dp)) != NULL)
    {
      int e_pid;

      if(sscanf(dir_entry->d_name,"%d",&e_pid))
      {
        FILE *fp;
        char str[STR_LEN];

        strcpy(str,"/proc/");
        strcat(str,dir_entry->d_name);
        strcat(str,"/stat");
        if((fp = fopen(str,"r")) != NULL)
        {
          char substr[STR_LEN];

          fgets(str,STR_LEN,fp);
          fclose(fp);
          if(sscanf(str,"%*u %s",substr))
          {
            if(!strcmp(substr,process_name))
              if(mode)
                proc_id = e_pid;
              else
                proc_id++;
          }
        }
      }
    }
    closedir(dp);
  }
  return proc_id;
}

/***************************************************************************
*
***************************************************************************/
void comp_net_us_ll(tus_ll *p,char *buf)
{
  char str[STR_LEN_LN + 1];
  register int x;

  sprintf(str,":%x\n",p->flags);
  strcat(buf,str);

  sprintf(str,":%s\n",p->user);
  strcat(buf,str);

  sprintf(str,":%s\n",p->email);
  strcat(buf,str);

  sprintf(str,":%x\n",p->tt_lim);
  strcat(buf,str);

  for(x = 0;x < DT_LIM_SIZE;x++)
  {
    sprintf(str,":%x %x %x %x %x\n",
      p->dt_lim[x].d_fr,
      p->dt_lim[x].d_to,
      p->dt_lim[x].t_fr,
      p->dt_lim[x].t_to,
      p->dt_lim[x].limit);
    strcat(buf,str);
  }

  sprintf(str,":%x %x %x\n",
    p->expiry_date.day,
    p->expiry_date.month,
    p->expiry_date.year);
  strcat(buf,str);

  sprintf(str,":%x\n",p->nice_factor);
  strcat(buf,str);

  sprintf(str,":%x\n",p->t_ul_lim);
  strcat(buf,str);

  sprintf(str,":%x\n",p->t_dl_lim);
  strcat(buf,str);

  sprintf(str,":%x\n",p->d_ul_lim);
  strcat(buf,str);

  sprintf(str,":%x\n",p->d_dl_lim);
  strcat(buf,str);
}

void comp_net_us_hr(tus_hr *p,char *buf)
{
  char str[STR_LEN_LN + 1];

  sprintf(str,":%x\n",p->flags);
  strcat(buf,str);

  sprintf(str,":%s\n",p->admin_email);
  strcat(buf,str);

  sprintf(str,":%x\n",p->sleep_time);
  strcat(buf,str);

  sprintf(str,":%x\n",p->grace_time);
  strcat(buf,str);

  sprintf(str,":%x\n",p->save_time);
  strcat(buf,str);

  sprintf(str,":%x\n",p->min_free_dialins);
  strcat(buf,str);

  sprintf(str,":%x\n",p->act_timeout);
  strcat(buf,str);

  comp_net_us_ll(&(p->default_us),buf);
}

/***************************************************************************
*
***************************************************************************/
void decomp_net_us_ll(tus_ll *p,int c,char *ins)
{
  char *str;

  if(ins[0] != ':')
    return;
  str = ins + 1;
  switch(c)
  {
    case 0:
      sscanf(str,"%x",&(p->flags));
      break;
    case 1:
      sscanf(str,"%s",(p->user));
      break;
    case 2:
      sscanf(str,"%s",(p->email));
      break;
    case 3:
      {
        register int i;

        sscanf(str,"%x",&(p->tt_lim));
        for(i = 0;i < DT_LIM_SIZE;i++)
          p->dt_lim[i].limit = -1;
      }
      break;
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
    case 10:
    case 11:
      {
        register int i;

        for(i = 0;i < DT_LIM_SIZE;i++)
          if(p->dt_lim[i].limit == -1)
          {
            int w,x,y,z;

            sscanf(str,"%x %x %x %x %x",
              &w,&x,&y,&z,&(p->dt_lim[i].limit));
            p->dt_lim[i].d_fr = w;
            p->dt_lim[i].d_to = x;
            p->dt_lim[i].t_fr = y;
            p->dt_lim[i].t_to = z;
            break;
          }
      }
      break;
    case 12:
      {
        int a,b;

        sscanf(str,"%x %x %x",&a,&b,&(p->expiry_date.year));
        p->expiry_date.day = a;
        p->expiry_date.month = b;
      }
      break;
    case 13:
      sscanf(str,"%x",&(p->nice_factor));
      break;
    case 14:
      sscanf(str,"%x",&(p->t_ul_lim));
      break;
    case 15:
      sscanf(str,"%x",&(p->t_dl_lim));
      break;
    case 16:
      sscanf(str,"%x",&(p->d_ul_lim));
      break;
    case 17:
      sscanf(str,"%x",&(p->d_dl_lim));
      break;
  }
}

void decomp_net_us_hr(tus_hr *p,int c,char *ins)
{
  char *str;

  if(ins[0] != ':')
    return;
  str = ins + 1;
  switch(c)
  {
    case 0:
      sscanf(str,"%x",&(p->flags));
      break;
    case 1:
      sscanf(str,"%s",p->admin_email);
      break;
    case 2:
      sscanf(str,"%x",&(p->sleep_time));
      break;
    case 3:
      sscanf(str,"%x",&(p->grace_time));
      break;
    case 4:
      sscanf(str,"%x",&(p->save_time));
      break;
    case 5:
      sscanf(str,"%x",&(p->min_free_dialins));
      break;
    case 6:
      sscanf(str,"%x",&(p->act_timeout));
      break;
    default:
      decomp_net_us_ll(&(p->default_us),c - 7,ins);
      break;
  }
}

/***************************************************************************
*
***************************************************************************/
void comp_net_um_ll(tum_ll *p,char *buf)
{
  char str[STR_LEN_LN + 1]; 
    
  sprintf(str,":%s\n",p->user);
  strcat(buf,str);

  sprintf(str,":%x\n",p->no_active);
  strcat(buf,str);

  sprintf(str,":%x\n",p->grace_countdown);
  strcat(buf,str);

  sprintf(str,":%x\n",p->last_active);
  strcat(buf,str);

  sprintf(str,":%x\n",p->tt_used);
  strcat(buf,str);

  sprintf(str,":%x\n",p->dt_zone_limit);
  strcat(buf,str);

  sprintf(str,":%x\n",p->dt_set_limit);
  strcat(buf,str);

  sprintf(str,":%x\n",p->dt_used);
  strcat(buf,str);

  sprintf(str,":%x\n",p->t_ul);
  strcat(buf,str);

  sprintf(str,":%x\n",p->t_dl);
  strcat(buf,str);

  sprintf(str,":%x\n",p->d_ul);
  strcat(buf,str);

  sprintf(str,":%x\n",p->d_dl);
  strcat(buf,str);
}


/***************************************************************************
*
***************************************************************************/
void decomp_net_um_ll(tum_ll *p,int c,char *ins)
{
  char *str;
      
  if(ins[0] != ':')
    return;
  str = ins + 1;
  switch(c)
  {
    case 0:
      sscanf(str,"%s",p->user);
      break;
    case 1:
      sscanf(str,"%x",&(p->no_active));
      break;
    case 2:
      sscanf(str,"%x",&(p->grace_countdown));
      break;
    case 3:
      sscanf(str,"%x",&(p->last_active));
      break;
    case 4:
      sscanf(str,"%x",&(p->tt_used));
      break;
    case 5:
      sscanf(str,"%x",&(p->dt_zone_limit));
      break;
    case 6:
      sscanf(str,"%x",&(p->dt_set_limit));
      break;
    case 7:
      sscanf(str,"%x",&(p->dt_used));
      break;
    case 8:
      sscanf(str,"%x",&(p->t_ul));
      break;
    case 9:
      sscanf(str,"%x",&(p->t_dl));
      break;
    case 10:
      sscanf(str,"%x",&(p->d_ul));
      break;
    case 11:
      sscanf(str,"%x",&(p->d_dl));
      break;
  }
}
